﻿using DbCommon.BusinessCore.DbCore;
using DbCommon.Enties.DbModels;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DbCommon.BusinessCore.BaseCore
{
    public class LocationPropertyManager : DbContext<LocationProperty>
    {
        public void Init()
        {
            if (!Db.DbMaintenance.IsAnyTable("t_wh_locationProperty", false))
            {
                //Create tables
                Db.CodeFirst.InitTables(typeof(LocationProperty));
            }

            string tableName = Db.EntityMaintenance.GetTableName<LocationProperty>();

            var dbColInfo = new SqlSugar.DbColumnInfo() { DbColumnName = "RowIndex", DataType = "int", DefaultValue = "0", IsNullable = true };
            if (!Db.DbMaintenance.IsAnyColumn(tableName, dbColInfo.DbColumnName, false))
            {
                Db.DbMaintenance.AddColumn(tableName, dbColInfo);
            }

            dbColInfo.DbColumnName = "ColumnIndex";
            if (!Db.DbMaintenance.IsAnyColumn(tableName, dbColInfo.DbColumnName, false))
            {
                Db.DbMaintenance.AddColumn(tableName, dbColInfo);
            }

            dbColInfo.DbColumnName = "LayerIndex";
            if (!Db.DbMaintenance.IsAnyColumn(tableName, dbColInfo.DbColumnName, false))
            {
                Db.DbMaintenance.AddColumn(tableName, dbColInfo);
            }
        }

        public List<LocationProperty> QueryDataList(string wHouse)
        {
            return Db.Queryable<LocationProperty>()
                .Where(d => d.WareHouseCode == wHouse)
                .OrderBy(d => d.LocationCode.Length)
                .OrderBy(d => d.LocationCode)
                .ToList();
        }

        public List<LocationProperty> QueryDataList_Area(string area)
        {
            return Db.Queryable<LocationProperty>()
                .Where(d => d.Area == area)
                .OrderBy(d => d.LocationCode.Length)
                .OrderBy(d => d.LocationCode)
                .ToList();
        }

        public string QueryWareHouse(string locationCode)
        {
            return Db.Queryable<LocationProperty>()
                .Where(d => d.LocationCode == locationCode)
                .Select(f => f.WareHouseCode)
                .First();
        }

        public DataTable QueryAreaDistinct()
        {
            return Db.Queryable<LocationProperty>()
                .OrderBy(d => d.Area)
                .Select(d => d.Area)
                .Distinct()
                .ToDataTable();
        }

        public DataTable QueryAreaDistinct(string area)
        {
            return Db.Queryable<LocationProperty>()
                .Where(d => d.Area.Contains(area))
                .OrderBy(d => d.Area)
                .Select(d => d.Area)
                .Distinct()
                .ToDataTable();
        }

        public DataTable QueryAreaDistinct(string[] areaArray)
        {
            return Db.Queryable<LocationProperty>()
                .Where(d => areaArray.Contains(d.Area))
                .OrderBy(d => d.Area)
                .Select(d => d.Area)
                .Distinct()
                .ToDataTable();
        }

        public bool CheckLocation(LocationProperty data, params LocationStatus[] locationStatuses)
        {
            if (data == null) return false;
            //检查出库的线路上是否都是空状态（障碍检测）
            int count = Db.Queryable<LocationProperty>()
                .Count(d => d.WareHouseCode == data.WareHouseCode
                    && !locationStatuses.Contains(d.Status)
                    && string.IsNullOrEmpty(d.CurrentTaskID)
                    && d.ColumnIndex == data.ColumnIndex
                    && d.RowIndex < data.RowIndex);

            return count == 0;
        }

        public bool CheckLocation(LocationProperty data)
        {
            if (data == null) return false;
            //检查出库的线路上是否都是空状态（障碍检测）
            int count = Db.Queryable<LocationProperty>()
                .Count(d => d.WareHouseCode == data.WareHouseCode
                    && d.Status != LocationStatus.Empty
                    && string.IsNullOrEmpty(d.CurrentTaskID)
                    && d.ColumnIndex == data.ColumnIndex
                    && d.RowIndex < data.RowIndex);

            return count == 0;
        }

        public bool CheckLocation(string locationCode)
        {
            var data = Db.Queryable<LocationProperty>()
                .Where(d => d.LocationCode == locationCode)
                .First();

            if (data == null)
            {
                return false;
            }

            return CheckLocation(data);
        }

        public string QueryLocation_Out(string wHouse, LocationStatus status, string material = null)
        {
            var locationList = QueryLocation_Outable(wHouse, material, status).ToList();
            return locationList.FirstOrDefault(d => CheckLocation(d))?.LocationCode;
        }

        public List<LocationProperty> QueryLocation_Out_DataList(string wHouse, LocationStatus status, string material = null)
        {
            var locationList = QueryLocation_Outable(wHouse, material, status).ToList();
            return locationList.Where(d => CheckLocation(d)).ToList();
        }

        private SqlSugar.ISugarQueryable<LocationProperty> QueryLocation_Outable(string wHouse, string material, params LocationStatus[] statusArray)
        {
            var queryable = Db.Queryable<LocationProperty>()
                .Where(d => d.WareHouseCode == wHouse && statusArray.Contains(d.Status) && string.IsNullOrEmpty(d.CurrentTaskID));

            queryable = material == null ? queryable : queryable.Where(d => d.MaterialId == material);

            return queryable
                //.OrderBy(d => d.LocationCode.Length)
                //.OrderBy(d => d.LocationCode)
                .OrderBy(d => d.ColumnIndex, SqlSugar.OrderByType.Asc)
                .OrderBy(d => d.RowIndex, SqlSugar.OrderByType.Asc)
                .OrderBy(d => d.ModifyTime, SqlSugar.OrderByType.Asc);
        }

        private SqlSugar.ISugarQueryable<LocationProperty> QueryLocation_Inable(string wHouse, params LocationStatus[] statusArray)
        {
            return Db.Queryable<LocationProperty>()
            .Where(d => d.WareHouseCode == wHouse && statusArray.Contains(d.Status) && string.IsNullOrEmpty(d.CurrentTaskID))
            //.OrderBy(d => d.LocationCode.Length)
            //.OrderBy(d => d.LocationCode)
            .OrderBy(d => d.ColumnIndex, SqlSugar.OrderByType.Asc)
            .OrderBy(d => d.RowIndex, SqlSugar.OrderByType.Desc);
        }

        public List<LocationProperty> QueryLocation_In_DataList(string wHouse, LocationStatus status = LocationStatus.Empty)
        {
            var locationList = QueryLocation_Inable(wHouse, status).ToList();
            return locationList.Where(d => CheckLocation(d)).ToList();
        }

        public string QueryLocation_In(string wHouse)
        {
            return QueryLocation_In(wHouse, LocationStatus.Empty);
        }

        public string QueryLocation_In(string wHouse, params LocationStatus[] statusArray)
        {
            var locationList = QueryLocation_Inable(wHouse, statusArray).ToList();
            return locationList.FirstOrDefault(d => CheckLocation(d, statusArray))?.LocationCode;
        }

        public string QueryLocation_In_Transfer(string wHouse, int colIndex, params LocationStatus[] statusArray)
        {
            var locationList = Db.Queryable<LocationProperty>()
            .Where(d => d.WareHouseCode == wHouse && statusArray.Contains(d.Status) && d.ColumnIndex != colIndex)
            .OrderBy(d => SqlSugar.SqlFunc.Abs(d.ColumnIndex - colIndex), SqlSugar.OrderByType.Asc)
            .OrderBy(d => d.RowIndex, SqlSugar.OrderByType.Desc)
            .ToList();

            return locationList.FirstOrDefault(d => CheckLocation(d, statusArray))?.LocationCode;
        }

        public bool GetTransfer(string locationCode, out string location1, out string location2)
        {
            location1 = null;
            location2 = null;

            var data = Db.Queryable<LocationProperty>()
                .Where(d => d.LocationCode == locationCode)
                .First();

            if (data == null)
            {
                return false;
            }

            var queryable = Db.Queryable<LocationProperty>()
                .Where(d => d.WareHouseCode == data.WareHouseCode && d.Status == LocationStatus.Filled && d.ColumnIndex == data.ColumnIndex && d.RowIndex < data.RowIndex)
                .OrderBy(d => d.RowIndex, SqlSugar.OrderByType.Asc)
                .First();

            if (queryable == null)
            {
                return false;
            }

            location1 = queryable.LocationCode;
            location2 = QueryLocation_In_Transfer(data.WareHouseCode, data.ColumnIndex, LocationStatus.Empty);

            return !string.IsNullOrEmpty(location1) && !string.IsNullOrEmpty(location2);
        }

        public bool UpdateNotLocked(LocationProperty data)
        {
            return Db.Updateable(data)
                .Where(d => d.LocationCode == data.LocationCode
                && d.Status != LocationStatus.Locked
                && d.Status != LocationStatus.Exception)
                .ExecuteCommand() > 0;
        }

        public bool UpdateWithTaskId(LocationProperty data, string taskId = null)
        {
            var updateable = Db.Updateable(data)
                .Where(d => d.LocationCode == data.LocationCode);

            updateable = taskId == null ?
                updateable.Where(d => string.IsNullOrEmpty(d.CurrentTaskID)) :
                updateable.Where(d => d.CurrentTaskID == taskId);

            return updateable.ExecuteCommand() > 0;
        }
    }
}
