﻿using DbCommon.BusinessCore.DbCore;
using DbCommon.Enties.DbModels;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DbCommon.BusinessCore.BaseCore
{
    public class WareHouseManager : DbContext<WareHouse>
    {
        public void Init()
        {

            if (!Db.DbMaintenance.IsAnyTable("t_wh_warehouse", false))
            {
                //Create tables
                Db.CodeFirst.InitTables(typeof(WareHouse));
            }

            string tableName = Db.EntityMaintenance.GetTableName<WareHouse>();

            var dbColInfo = new SqlSugar.DbColumnInfo() { DbColumnName = "TargetWareHouseCode", DataType = "nvarchar", Length = 1000, IsNullable = true };
            if (!Db.DbMaintenance.IsAnyColumn(tableName, dbColInfo.DbColumnName, false))
            {
                Db.DbMaintenance.AddColumn(tableName, dbColInfo);
            }

            dbColInfo.DbColumnName = "SourceWareHouseCode";
            if (!Db.DbMaintenance.IsAnyColumn(tableName, dbColInfo.DbColumnName, false))
            {
                Db.DbMaintenance.AddColumn(tableName, dbColInfo);
            }
        }

        public DataTable QueryWareHouse()
        {
            return CurrentDb.AsQueryable()
                .OrderBy(d => d.WareHouseProperty, SqlSugar.OrderByType.Asc)
                .ToDataTable();
        }

        public DataTable QueryWareHouse(string filter)
        {
            return CurrentDb.AsQueryable()
                .Where(d => d.WareHouseCode.Contains(filter))
                .OrderBy(d => d.WareHouseProperty, SqlSugar.OrderByType.Asc)
                .ToDataTable();
        }

        public DataTable QueryWareHouse(string[] filters)
        {
            return CurrentDb.AsQueryable()
                .Where(d => filters.Contains(d.WareHouseCode))
                .OrderBy(d => d.WareHouseProperty, SqlSugar.OrderByType.Asc)
                .ToDataTable();
        }

        public List<WareHouse> QueryWareHouseList(string[] filters)
        {
            return CurrentDb.AsQueryable()
                .Where(d => filters.Contains(d.WareHouseCode))
                .ToList()
                .OrderBy(d => filters.ToList().IndexOf(d.WareHouseCode))
                .ToList();
        }
    }
}
