﻿using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Collections.Generic;
using System.Xml.Serialization;

namespace DbCommon
{
    internal class Config
    {
        private static Config m_instance;
        private static readonly object m_locker = new object();
        public static Config Instance
        {
            get
            {
                if (m_instance == null)
                {
                    lock (m_locker)
                    {
                        if (m_instance == null)
                        {
                            m_instance = new Config();
                        }
                    }
                }
                return m_instance;
            }
        }

        public static DbCommonConfig DbConfig = new DbCommonConfig();

        public string WmsConnString { get { return DbConfig.WmsConnString; } }
        public SqlSugar.DbType WmsDbType { get { return DbConfig.WmsDbType; } }

        private Config()
        {
            try
            {
                string s = System.Reflection.Assembly.GetExecutingAssembly().Location;
                FileInfo finfo = new FileInfo(s);

                string currPath = finfo.DirectoryName;
                string configDir = Path.Combine(currPath, "PMS", "Config");
                string configFile = Path.Combine(configDir, "DbCommonConfig.xml");

                DirectoryInfo dir = new DirectoryInfo(configDir);
                if (!dir.Exists)
                {
                    dir.Create();
                }

                XmlSerializer serializer = new XmlSerializer(DbConfig.GetType());
                if (!File.Exists(configFile))
                {
                    using (TextWriter tw = new StreamWriter(configFile))
                    {
                        serializer.Serialize(tw, DbConfig);
                    }
                }
                else
                {
                    using (TextReader tr = new StreamReader(configFile))
                    {
                        DbConfig = (DbCommonConfig)serializer.Deserialize(tr);
                    }
                }
            }
            catch (Exception)
            {
            }
        }
    }

    public class DbCommonConfig
    {
        public string WmsConnString { get; set; } = "server=localhost;uid=sa;pwd=123;database=WMS";
        public SqlSugar.DbType WmsDbType { get; set; } = SqlSugar.DbType.SqlServer;
    }
}
