﻿using HslCommunication.LogNet;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Logger
{
    public class CLog
    {
        public event Action<int, string> OnLog;

        private static string currDir;
        private static CLog m_cLog = new CLog();
        private ConcurrentDictionary<string, ILogNet> m_logDictionary = new ConcurrentDictionary<string, ILogNet>();

        public static CLog Instance
        {
            get { return m_cLog; }
        }

        public string LogDir { get { return currDir; } }
        public ILogNet SystemLog { get { return GetLog("System"); } }
        public ILogNet TaskLog { get { return GetLog("Task"); } }
        public ILogNet DeviceLog { get { return GetLog("Device"); } }
        public ILogNet TibcoLog { get { return GetLog("Tibco"); } }

        private CLog()
        {
            string s = System.Reflection.Assembly.GetExecutingAssembly().Location;
            FileInfo finfo = new FileInfo(s);

            string currPath = finfo.DirectoryName;
            currDir = Path.Combine(currPath, "PMS", "Log");

            DirectoryInfo dir = new DirectoryInfo(currDir);
            if (!dir.Exists)
            {
                dir.Create();
            }
        }

        public ILogNet GetLog(string file)
        {
            if (!m_logDictionary.ContainsKey(file))
            {
                string filePath = Path.Combine(currDir, file);
                var logNet = new LogNetDateTime(filePath, GenerateMode.ByEveryDay);
                logNet.SetMessageDegree(HslMessageDegree.DEBUG);
                logNet.BeforeSaveToFile += LogNet_BeforeSaveToFile;

                m_logDictionary.AddOrUpdate(file, logNet, (key, value) => logNet);
            }

            return m_logDictionary[file];
        }

        public ILogNet GetAgvLog(int agvId)
        {
            if(agvId > 0)
            {
                return GetLog($"AGV{agvId}");
            }
            return TaskLog;
        }

        private void LogNet_BeforeSaveToFile(object sender, HslEventArgs e)
        {
            if (OnLog != null)
            {
                OnLog((int)e.HslMessage.Degree, e.HslMessage.ToString());
            }

            if (sender is ILogNet log)
            {
                DeleteLogs(log.GetExistLogFileNames());
            }
        }

        private void DeleteLogs(string[] files)
        {
            try
            {
                int controlFileQuantity = 15;

                if (files.Length > controlFileQuantity)
                {
                    List<FileInfo> fileInfos = new List<FileInfo>();
                    for (int i = 0; i < files.Length; i++)
                    {
                        fileInfos.Add(new FileInfo(files[i]));
                    }

                    fileInfos.Sort(new Comparison<FileInfo>((m, n) =>
                    {
                        return m.CreationTime.CompareTo(n.CreationTime);
                    }));


                    for (int i = 0; i < fileInfos.Count - controlFileQuantity; i++)
                    {
                        File.Delete(fileInfos[i].FullName);
                    }
                }
            }
            catch
            {

            }
        }
    }
}
