﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskBookEvent
{

    public class AsyncBehaviorPackEStop : AsyncBehavior
    {
        public string EStopColumeName { get; set; } = "PLC急停";

        public override bool CustomOperator(TaskData taskData, StepData stepInfo)
        {
            ////取任务第一个任务点
            //string wareHouseId = taskData.StepList.OrderBy(d => d.StepID).FirstOrDefault().WareHouseID;

            var routeConfig = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID);
            if (routeConfig == null)
            {
                Log(taskData, stepInfo, $"查找库位配置信息失败");
                return false;
            }
            if (string.IsNullOrEmpty(routeConfig.PlcIpAddr))
            {
                Log(taskData, stepInfo, $"PlcIp地址为空");
                return false;
            }

            string eStopAddr = routeConfig.LocationCode.GetLocationMember(EStopColumeName);
            if (string.IsNullOrEmpty(eStopAddr))
            {
                Log(taskData, stepInfo, $"{EStopColumeName}地址为空");
                return false;
            }

            var tempResult = routeConfig.PlcReadWrite.ReadBool(eStopAddr);
            if (!tempResult.IsSuccess)
            {
                Log(taskData, stepInfo, $"读取失败 {EStopColumeName}{eStopAddr} {tempResult.Message}");
                return false;
            }

            bool eStop = tempResult.IsSuccess && tempResult.Content;
            bool realEStop = eStop;

            bool needPause = realEStop;
            if (needPause)
            {
                bool result = Crms.PmsApi.CarrierPause(taskData.Carrier);
                Log(taskData, stepInfo, $"暂停小车{result.ToChineseString()}{EStopColumeName}{eStopAddr}={eStop}");
            }

            bool needResume = !realEStop;
            if (needResume)
            {
                bool result = Crms.PmsApi.CarrierResume(taskData.Carrier);
                Log(taskData, stepInfo, $"恢复小车{result.ToChineseString()}{EStopColumeName}{eStopAddr}={eStop}");
            }

            return false;
        }
    }
}
