﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.Extenions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Models.Crms;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class AsyncBehaviorRollerAllowIn : AsyncBehavior
    {
        public string AllowInColumnName { get; set; } = "AGV滚筒已启动";

        public override bool CustomOperator(TaskData taskData, StepData stepInfo)
        {
            var routeConfig = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID);
            if (routeConfig == null)
            {
                Log(taskData, stepInfo, "查找库位配置信息失败");
                return false;
            }
            if (string.IsNullOrEmpty(routeConfig.PlcIpAddr))
            {
                Log(taskData, stepInfo, "PlcIp地址为空");
                return false;
            }

            var agvData = Crms.PmsApi.GetCarrier(taskData.Carrier);
            if (agvData == null)
            {
                Log(taskData, stepInfo, $"获取AGV数据失败");
                return false;
            }

            bool allowIn = agvData.LogicBits.GetBitVaule(7);
            string allowInAddr = routeConfig.LocationCode.GetLocationMember(AllowInColumnName);

            var result = routeConfig.PlcReadWrite.Write(allowInAddr, allowIn);
            Log(taskData, stepInfo, $"LogicBits{agvData.LogicBits} {AllowInColumnName} {allowInAddr}={allowIn} 写入{result.IsSuccess.ToChineseString()}");

            return false;
        }
    }
}
