﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.Extenions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Models.Crms;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class AsyncBehaviorRollerAllowIn_IO : AsyncBehavior
    {
        public string AllowInColumnName { get; set; } = "AGV滚筒已启动";

        public override bool CustomOperator(TaskData taskData, StepData stepInfo)
        {
            var routeConfig = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID);
            if (routeConfig == null)
            {
                Log(taskData, stepInfo, "查找库位配置信息失败");
                return false;
            }

            if (routeConfig.IOTransID <= 0)
            {
                Log(taskData, stepInfo, "获取库位对接盒ID失败");
                return false;
            }

            int id = routeConfig.IOTransID;
            if (!DeviceControl.Instance.Communication.InfoIODictionary.ContainsKey(id))
            {
                Log(taskData, stepInfo, $"获取库位对接盒{id}，没有此盒子数据，检查盒子是否连接网络");
                return false;
            }

            var infoIO = DeviceControl.Instance.Communication.InfoIODictionary[id];
            if (!infoIO.DataValid)
            {
                Log(taskData, stepInfo, $"获取库位对接盒{id}，盒子已掉线！");
                return false;
            }

            var agvData = Crms.PmsApi.GetCarrier(taskData.Carrier);
            if (agvData == null)
            {
                Log(taskData, stepInfo, $"获取AGV数据失败");
                return false;
            }

            bool allowIn = agvData.LogicBits.GetBitVaule(7);
            //if (stepInfo.AgvBehavior == 65350
            //    && stepInfo.Param1 > 0
            //    && allowIn)
            //{
            //    //执行的是设备->滚筒的上料任务
            //    allowIn = (stepInfo.Param1 & 0x5555) > 0;
            //}

            string allowInAddr = routeConfig.LocationCode.GetLocationMember(AllowInColumnName);

            var result = DeviceControl.Instance.SetOutputIO_ThenCheck(id, new string[] { allowInAddr }, new bool[] { allowIn }, out string msg);
            Log(taskData, stepInfo, $"LogicBits{agvData.LogicBits} ID{id} {AllowInColumnName} {allowInAddr}={allowIn} 写入{result.ToChineseString()} {msg}");

            return false;
        }
    }
}
