﻿using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Extenions;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class AsyncBehaviorRollerHxSend : AsyncBehavior
    {
        public override bool CustomOperator(TaskData taskData, StepData stepInfo)
        {
            var agvData = Crms.PmsApi.GetCarrier(taskData.Carrier);
            if (agvData == null)
            {
                Log(taskData, stepInfo, "获取AGV数据失败");
                return false;
            }

            //判断AGV滚筒是否已经转动
            var logicBits = agvData.LogicBits;
            bool rollerMoveing = logicBits.GetBitVaule(7);
            if (!rollerMoveing)
            {
                Log(taskData, stepInfo, $"等待AGV滚筒转动，LogicBits：{logicBits}");
                return false;
            }

            int id = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID).RollerID;

            var dict = DeviceControl.Instance.Communication.InfoRollerHxDictionary;
            if (!dict.ContainsKey(id))
            {
                Log(taskData, stepInfo, "获取滚筒台数据失败");
                return false;
            }

            var data = dict[id];
            if (data.DataValid
                && data.NoAction
                && data.Empty
                && data.AlarmCode == 0)
            {
                Log(taskData, stepInfo, $"滚筒台无动作{data.NoAction} 空状态{data.Empty} 报警{data.AlarmCode}");
                return true;
            }
            else
            {
                ushort actionCode = 0x02;
                Log(taskData, stepInfo, $"AGV滚筒已转动，LogicBits：{logicBits}，滚筒台无动作{data.NoAction} 空状态{data.Empty}，有效：{data.DataValid}，发送操作0x{actionCode:X2}");
                DeviceControl.Instance.RollerHxOperate(id, actionCode);
            }

            return false;
        }
    }
}
