﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Config;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class TaskBookBehavior_63007 : BaseTaskBookBehaviorV2
    {
        //LocationPropertyManager locationManager = new LocationPropertyManager();
        public TaskBookBehavior_63007(int behavior, string remark) : base(behavior, remark)
        {
        }

        public TaskBookBehavior_63007() : base(63007, "更改步骤库位")
        {
        }

        protected virtual bool CheckAllowEnter(string location)
        {
            //string agvRequestEnter = location.GetLocationMember("AGV请求进入");
            //if (!string.IsNullOrEmpty(agvRequestEnter))
            //{
            //    //如果有AGV请求，那任务所有工位都可以进入
            //    return true;
            //}

            string plcAllowEnter = location.GetLocationMember("PLC允许进入");
            if (string.IsNullOrEmpty(plcAllowEnter))
            {
                return true;
            }

            var config = ExcelConfig.Instance.GetRouteConfig(location);
            if (config == null
            || config.PlcReadWrite == null
            || config.PlcConnected == false)
            {
                return false;
            }

            var read = config.PlcReadWrite.ReadBool(plcAllowEnter);
            return read.IsSuccess && read.Content;
        }

        private bool CheckNoTask(string location, TaskData taskDetailInfo, List<TaskData> taskDataList)
        {
            //查找结束点是当前工位的任务
            var taskDataFind = taskDataList.FirstOrDefault(d =>
            {
                //不算自身任务
                if (d.TaskID == taskDetailInfo.TaskID) return false;

                var step = d.StepList?.OrderBy(s => s.StepID);
                var firstPos = step?.First()?.WareHouseID;
                var lastPos = step?.Last()?.WareHouseID;

                //不算放行任务
                if (firstPos != lastPos)
                {
                    if (firstPos == location)
                    {
                        return false;
                    }
                }

                //正在执行步骤的位置
                var currentPos = d.GetCurrentStepInfo()?.WareHouseID;
                return currentPos == location;
            });

            return taskDataFind == null;
        }

        private bool CheckNoAgv(string location)
        {
            var ignoreCheck = location.GetLocationMember("忽略检查工位AGV").ToValue<bool>();
            if (ignoreCheck)
            {
                return true;
            }

            var sDict = PackControl.Instance.StationDictionary;
            return sDict.ContainsKey(location) && !sDict[location].区域有Agv;
        }

        public override bool TaskBookOperator(TaskData taskDetailInfo, StepData stepInfo)
        {
            try
            {
                string allNextLocation = stepInfo.WareHouseID.GetLocationMember("下个工位");

                string[] nextLocations = allNextLocation.ToValueArray<string>();
                bool[] sAllowArray = new bool[nextLocations.Length];
                bool[] sNoTaskArray = new bool[nextLocations.Length];
                bool[] sNoAgvArray = new bool[nextLocations.Length];

                var taskDataList = Crms.PmsApi.GetTaskDataList();
                if (taskDataList == null)
                {
                    Log(taskDetailInfo, stepInfo, $"{allNextLocation} 获取所有任务失败");
                    return false;
                }

                StringBuilder stringBuilder = new StringBuilder();
                for (int i = 0; i < nextLocations.Length; i++)
                {
                    var location = nextLocations[i];
                    sAllowArray[i] = CheckAllowEnter(location);
                    sNoTaskArray[i] = CheckNoTask(location, taskDetailInfo, taskDataList);
                    sNoAgvArray[i] = CheckNoAgv(location);

                    stringBuilder.Append($"{location} 允许进入{sAllowArray[i]} 无任务{sNoTaskArray[i]} 无小车{sNoAgvArray[i]}; ");
                }

                string nextLocation = null;
                for (int i = 0; i < nextLocations.Length; i++)
                {
                    var location = nextLocations[i];

                    if (sAllowArray[i] && sNoTaskArray[i] && sNoAgvArray[i])
                    {
                        nextLocation = location;
                        break;
                    }
                }

                if (string.IsNullOrEmpty(nextLocation))
                {
                    Log(taskDetailInfo, stepInfo, $"{allNextLocation} 中未找到可以分配的工位 【分配信息{stringBuilder}】");
                    return false;
                }

                Log(taskDetailInfo, stepInfo, $"{allNextLocation} 中已找到可以分配的工位 {nextLocation} 【分配信息{stringBuilder}】");

                //更改库位
                int endStepId = taskDetailInfo.StepList.Max(d => d.StepID);
                var result = Crms.PmsApi.UpdateStepWarehouse(taskDetailInfo.TaskID, endStepId, nextLocation);
                Log(taskDetailInfo, stepInfo, $"更改库位{result.ToChineseString()}");

                return result;
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }
    }
}
