﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.WebApi;
using ProjectManagementSystem.Common.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Models.Crms;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class TaskBookBehavior_63021 : BaseTaskBookBehaviorV2
    {
        AgvMaterialBindingManager agvMaterialBindingManager = new AgvMaterialBindingManager();

        public TaskBookBehavior_63021() : base(63021, "记录PACK物料")
        {
        }

        public override bool TaskBookOperator(TaskData taskDetailInfo, StepData stepInfo)
        {
            try
            {
                var routeConfig = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID);
                if (routeConfig == null)
                {
                    Log(taskDetailInfo, stepInfo, "查找库位配置信息失败");
                    return false;
                }
                if (string.IsNullOrEmpty(routeConfig.PlcIpAddr))
                {
                    Log(taskDetailInfo, stepInfo, "PlcIp地址为空");
                    return false;
                }
                if (routeConfig.PlcConnected)
                {
                    Log(taskDetailInfo, stepInfo, "PLC未连接");
                    return false;
                }
                //var plc = routeConfig.PlcReadWrite;

                string plcMaterialAddr = routeConfig.LocationCode.GetLocationMember("PLC物料");
                string agvMaterialAddr = routeConfig.LocationCode.GetLocationMember("AGV物料");

                if (!string.IsNullOrEmpty(plcMaterialAddr) && !string.IsNullOrEmpty(agvMaterialAddr))
                {
                    //读取PLC物料信号
                    var tempResult = routeConfig.PlcReadWrite.ReadInt16(plcMaterialAddr);

                    if(!tempResult.IsSuccess)
                    {
                        Log(taskDetailInfo, stepInfo, $"读取{tempResult.IsSuccess.ToChineseString()}({"PLC物料"}{plcMaterialAddr}) {tempResult.Message}");
                        return false;
                    }

                    //回写PLC物料信号
                    routeConfig.PlcReadWrite.Write(agvMaterialAddr, tempResult.Content);

                    //记录物料
                    var data = new AgvMaterialBinding();
                    data.Id = taskDetailInfo.Carrier;
                    data.Material = tempResult.Content.ToString();
                    var result = agvMaterialBindingManager.Save(data);

                    Log(taskDetailInfo, stepInfo, $"保存PLC物料{result.ToChineseString()} {data.Material}");
                    return result;
                }
                else
                {
                    Log(taskDetailInfo, stepInfo, $"没有配置物料地址，默认返回成功");
                    return true;
                }
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }
    }
}
