﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.WebApi;
using ProjectManagementSystem.Common.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Models.Crms;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class TaskBookBehavior_63022 : BaseTaskBookBehaviorV2
    {
        AgvMaterialBindingManager agvMaterialBindingManager = new AgvMaterialBindingManager();

        public TaskBookBehavior_63022() : base(63022, "写入PACK物料")
        {
        }

        public override bool TaskBookOperator(TaskData taskDetailInfo, StepData stepInfo)
        {
            try
            {
                var routeConfig = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID);
                if (routeConfig == null)
                {
                    Log(taskDetailInfo, stepInfo, "查找库位配置信息失败");
                    return false;
                }
                if (string.IsNullOrEmpty(routeConfig.PlcIpAddr))
                {
                    Log(taskDetailInfo, stepInfo, "PlcIp地址为空");
                    return false;
                }
                if (routeConfig.PlcConnected)
                {
                    Log(taskDetailInfo, stepInfo, "PLC未连接");
                    return false;
                }
                var plc = routeConfig.PlcReadWrite;

                string plcMaterialAddr = routeConfig.LocationCode.GetLocationMember("PLC物料");
                string agvMaterialAddr = routeConfig.LocationCode.GetLocationMember("AGV物料");

                if (!string.IsNullOrEmpty(plcMaterialAddr) && !string.IsNullOrEmpty(agvMaterialAddr))
                {
                    //查询AGV物料信号

                    var data = agvMaterialBindingManager.GetById(taskDetailInfo.Carrier);
                    if(data == null)
                    {
                        Log(taskDetailInfo, stepInfo, $"获取AGV物料信息失败，默认返回成功");
                        return true;
                    }

                    //回写PLC物料信号
                    var result = routeConfig.PlcReadWrite.Write(agvMaterialAddr, data.Material.ToValue<short>());

                    Log(taskDetailInfo, stepInfo, $"写入{result.IsSuccess.ToChineseString()}({"AGV物料"}{agvMaterialAddr}写入{data.Material}) {result.Message}");
                    return result.IsSuccess;
                }
                else
                {
                    Log(taskDetailInfo, stepInfo, $"没有配置物料地址，默认返回成功");
                    return true;
                }
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }
    }
}
