﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.WebApi;
using ProjectManagementSystem.Common.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Core;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class TaskBookBehavior_63031 : BaseTaskBookBehaviorV2
    {
        public TaskBookBehavior_63031() : base(63031, "AGV->滚筒 等待滚筒转动后启动AGV")
        {
        }

        public override bool TaskBookOperator(TaskData taskData, StepData stepInfo)
        {
            try
            {

                //检测滚筒台是不是正常空状态
                int id = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID).RollerID;

                var dict = DeviceControl.Instance.Communication.InfoRollerHxDictionary;
                if (!dict.ContainsKey(id))
                {
                    Log(taskData, stepInfo, $"获取滚筒台{id}数据失败");
                    return false;
                }

                var data = dict[id];
                if (!data.DataValid)
                {
                    Log(taskData, stepInfo, $"获取滚筒台{id}数据无效，已掉线");
                    return false;
                }

                if (data.AlarmCode > 0)
                {
                    Log(taskData, stepInfo, $"滚筒台{id}报警{data.AlarmCode}");
                    return false;
                }

                if (!data.Empty)
                {
                    Log(taskData, stepInfo, $"滚筒台{id}不是空状态");
                    return false;
                }

                if (data.Rotating)
                {
                    Log(taskData, stepInfo, $"滚筒台{id}已动作");
                    return true;
                }

                //启动滚筒
                ushort actionCode = 0x01;

                Log(taskData, stepInfo, $"{data.ToSimpleString()}，发送操作0x{actionCode:X2}");
                DeviceControl.Instance.RollerHxOperate(id, actionCode);
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }
    }
}
