﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.WebApi;
using ProjectManagementSystem.Common.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Core;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class TaskBookBehavior_63032 : BaseTaskBookBehaviorV2
    {
        public TaskBookBehavior_63032() : base(63032, "滚筒->AGV 等待AGV转动后启动滚筒储位")
        {
        }

        public override bool TaskBookOperator(TaskData taskData, StepData stepInfo)
        {
            try
            {
                //检测滚筒台是不是正常满状态
                int id = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID).RollerID;

                var dict = DeviceControl.Instance.Communication.InfoRollerHxDictionary;
                if (!dict.ContainsKey(id))
                {
                    Log(taskData, stepInfo, $"获取滚筒台{id}数据失败");
                    return false;
                }

                var data = dict[id];
                if (!data.DataValid)
                {
                    Log(taskData, stepInfo, $"获取滚筒台{id}数据无效，已掉线");
                    return false;
                }

                if (data.AlarmCode > 0)
                {
                    Log(taskData, stepInfo, $"滚筒台{id}报警{data.AlarmCode}");
                    return false;
                }

                if (!data.Full)
                {
                    Log(taskData, stepInfo, $"滚筒台{id}不是满状态");
                    return false;
                }

                var instance = new AsyncBehaviorRollerHxSend();

                instance.Behavior = stepInfo.PmsBehavior;
                instance.Interval = 1000;
                instance.TaskID = taskData.TaskID;
                instance.AgvId = taskData.Carrier;
                instance.MaxStepId = taskData.CurrentStepID + 1;

                instance.Init();
                instance.Start();

                return true;
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }
    }
}
