﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.WebApi;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessAlarmReport : ICustomProcess
    {
        private int[] ignoreAlarmIds = ExcelConfig.Instance.VcuAlarmInfoList?.Where(d => d.Ignore).Select(d => d.AlarmId).ToArray();

        private Dictionary<int, ResultAgvData> agvDataDict = new Dictionary<int, ResultAgvData>();

        public void CustomProcess(List<TaskData> taskDataList)
        {
            var agvDataList = Crms.PmsApi.GetAllCarrier();
            if (agvDataList == null) return;

            for (int i = 0; i < agvDataList.Count; i++)
            {
                var agvData = agvDataList[i];

                if (!agvDataDict.ContainsKey(agvData.AgvID))
                {
                    var result = Report(taskDataList, agvData);

                    agvDataDict.Add(agvData.AgvID, agvData);
                }
                else
                {
                    var lastAgvData = agvDataDict[agvData.AgvID];

                    var lastAlarm = lastAgvData.AlarmList?
                        .Where(d => d.alarmLevel > 1)?
                        .OrderByDescending(d => d.alarmLevel)?
                        .FirstOrDefault()?
                        .alarmId ?? 0;

                    var alarm = agvData.AlarmList?
                        .Where(d => d.alarmLevel > 1)?
                        .OrderByDescending(d => d.alarmLevel)?
                        .FirstOrDefault()?
                        .alarmId ?? 0;

                    if (alarm != lastAlarm)
                    {
                        var result = Report(taskDataList, agvData);

                    }

                    agvDataDict[agvData.AgvID] = agvData;
                }
            }
        }

        public bool Report(List<TaskData> taskDataList, ResultAgvData agvData)
        {
            var alarm = agvData.AlarmList?
                .Where(d => d.alarmLevel > 1)?
                .OrderByDescending(d => d.alarmLevel)?
                .FirstOrDefault()?
                .alarmId ?? 0;

            if (ignoreAlarmIds?.Contains(alarm) == true) return true;

            //没有任务的车不上报
            var taskData = taskDataList.FirstOrDefault(d => d.Carrier == agvData.AgvID);
            //if (taskData == null) return true;

            var alarmInfo = ExcelConfig.Instance.GetVcuAlarmInfo(alarm);
            var alarmDescribe = alarmInfo == null ? "" : alarmInfo.AlarmName;

            var result = CustomerApi.AgvAlarm(alarmDescribe, agvData.AgvID.ToString(), taskData?.TaskID, out string content);
            if(!result)
            {
                CLog.Instance.SystemLog.WriteInfo($"{taskData?.TaskID} {agvData.AgvID}# {alarm} {alarmDescribe} 上报报警信息失败 {content}");
            }

            return result;
        }
    }
}
