﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.WebApi;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessMesReport : ICustomProcess
    {
        private int[] ignoreAlarmIds = ExcelConfig.Instance.VcuAlarmInfoList.Where(d => d.Ignore).Select(d => d.AlarmId).ToArray();

        public void CustomProcess(List<TaskData> taskDataList)
        {
            var agvDataList = Crms.PmsApi.GetAllCarrier();
            if (agvDataList == null) return;

            var pileList = DeviceApi.GetAllPile();
            if (pileList == null) return;

            //AGV状态：正常：1、故障: 2、离线：3
            //充电桩状态：正常：1、故障: 2、充电中: 4


            //构建AGV状态列表数据
            var packStationDict = PackControl.Instance.StationDictionary;
            var agvStatusList = agvDataList
                .Select(d =>
                {
                    string position = packStationDict.Values.FirstOrDefault(f => f.AGV编号 == d.AgvID)?.Id;

                    string status = "0";
                    if (d.Online)
                    {
                        var alarm = d.AlarmList?
                            .FirstOrDefault(f => f.alarmLevel > 1 && !ignoreAlarmIds.Contains(f.alarmId));

                        status = alarm == null ? "1" : "2";
                    }
                    else
                    {
                        status = "3";
                    }

                    var body = new
                    {
                        api = "AgvStatusAPI",
                        agvNumber = d.AgvID.ToString(),
                        agvPosition = position ?? "",
                        agvStatus = status,
                        agvPower = d.BatteryLevel.ToString(),
                        agvXCoordinate = d.MovementX.ToString(),
                        agvYCoordinate = d.MovementY.ToString()
                    };

                    return body;
                })
                .ToList();

            //构建AGV报警列表数据
            var agvAlarmList = agvDataList
                .Select(d =>
                {
                    var alarm = d.AlarmList?
                        .Where(f => f.alarmLevel > 1 && !ignoreAlarmIds.Contains(f.alarmId))
                        .OrderByDescending(f => f.alarmLevel)
                        .FirstOrDefault();

                    var alarmCode = alarm == null ? 0 : alarm.alarmId;
                    var alarmInfo = ExcelConfig.Instance.GetVcuAlarmInfo(alarmCode);
                    var alarmDescribe = alarmInfo == null ? "" : alarmInfo.AlarmName;
                    var alarmTimeStr = alarm == null ? DateTime.Now.ToString("G") : alarm.startTime?.ToString("G");

                    var body = new
                    {
                        api = "AgvAlarmAPI",
                        agvNumber = d.AgvID.ToString(),
                        alarmCode = alarmCode.ToString(),
                        alarmDescribe = alarmDescribe,
                        alarmTime = alarmTimeStr
                    };

                    return body;
                })
                //.Where(d => d != null)
                .ToList();

            //构建充电站列表数据
            var chargeList = pileList
                .Select(d =>
                {
                    string status = "0";

                    switch (d)
                    {
                        case PileV4VO p when !p.OnLine:
                            status = "3";
                            break;
                        case PileV4VO p when p.OnLine && (p.ChargingStatus == ChargingStatus.AutoCharging || p.ChargingStatus == ChargingStatus.ManulCharging):
                            status = "4";
                            break;
                        case PileV4VO p when p.OnLine && d.AlarmStatus == AlarmStatus.None:
                            status = "1";
                            break;
                        case PileV4VO p when p.OnLine && d.AlarmStatus != AlarmStatus.None:
                            status = "2";
                            break;
                        default:
                            break;
                    }

                    var body = new
                    {

                        api = "PileAPI",
                        dianPileNumber = d.ID.ToString(),
                        dianPilePosition = d.Name,
                        dianPileStatus = status
                    };

                    return body;
                })
                .ToList();

            CustomerMesApi.AgvStatusData(agvStatusList, out string _);
            CustomerMesApi.AgvAlarmData(agvAlarmList, out string _);
            CustomerMesApi.AgvChargePileData(chargeList, out string _);
        }
    }
}
