﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessPlcClear : ICustomProcess
    {
        private List<string> m_currentPosList = new List<string>();
        public void CustomProcess(List<TaskData> dataList)
        {
            m_currentPosList.Clear();
            for (int i = 0; i < dataList.Count; i++)
            {
                var data = dataList[i];

                string[] posArray = data.StepList?
                    .OrderBy(d => d.StepID)?
                    .Select(d => d.WareHouseID)?
                    .Distinct()?
                    .ToArray();

                if (posArray != null)
                {
                    for (int idx = 0; idx < posArray.Length; idx++)
                    {
                        m_currentPosList.Add(posArray[idx]);
                    }
                }
            }

            var RouteConfigList = ExcelConfig.Instance.RouteConfigList;
            for (int i = 0; i < RouteConfigList.Count; i++)
            {
                var routeConfig = RouteConfigList[i];

                //工位没有任务了，需要清零信号
                routeConfig.PlcIsNeedClear = (!string.IsNullOrEmpty(routeConfig.PlcIpAddr) || routeConfig.IOTransID > 0)&& !m_currentPosList.Contains(routeConfig.LocationCode);

                if(routeConfig.PlcIsNeedClear)
                {
                    Clear(routeConfig);
                }
            }
        }


        private void Clear(RouteConfig routeConfig)
        {
            var plcBehaviorList = ExcelConfig.Instance.PlcBehaviorList
                .Where(d => !string.IsNullOrEmpty(d.Remark) && d.Remark.Contains("初始化"));

            if (plcBehaviorList == null
                || plcBehaviorList.Count() == 0)
            {
                //string strLog = $"{routeConfig.LocationCode} IP地址：{routeConfig.PlcIpAddr}，无需清零地址";
                //CLog.Instance.SystemLog.WriteInfo(strLog);
                return;
            }

            foreach (var item in plcBehaviorList)
            {
                foreach (string addrDesc in item.DbAddressWrite)
                {
                    string addr = routeConfig.LocationCode.GetPlcTaskAddr(addrDesc);
                    if (string.IsNullOrEmpty(addr)) continue;

                    int id = routeConfig.IOTransID;
                    if (id > 0)
                    {
                        var readResult2 = DeviceControl.Instance.GetOutputIO(id, addr);
                        if (readResult2.Item1
                            && readResult2.Item2)
                        {
                            DeviceControl.Instance.SetOutputIO(id, addr, false);
                            string strLog = $"{routeConfig.LocationCode}，IO对接盒ID{id}，清零：{addr} ({addrDesc})";
                            CLog.Instance.SystemLog.WriteInfo(strLog);
                            Thread.Sleep(50);
                        }
                    }

                    var plc = routeConfig.PlcReadWrite;
                    if (plc != null && routeConfig.PlcConnected)
                    {
                        var readResult = plc.ReadBool(addr);
                        if (!readResult.IsSuccess
                            || (readResult.IsSuccess && readResult.Content != false))
                        {
                            var result = plc.Write(addr, false);
                            string strLog = $"{routeConfig.LocationCode}，{routeConfig.PlcIpAddr}，清零：{addr} ({addrDesc})，写入{result.IsSuccess.ToChineseString()}";
                            CLog.Instance.SystemLog.WriteInfo(strLog);
                        }
                    }

                }
            }
        }
    }
}
