﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.WebApi;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessPlcTask7614 : ICustomProcess
    {
        private List<RouteConfig> routeConfigs = ExcelConfig.Instance.RouteConfigList
            .Where(d => d.WareHouseCode == "模组")
            .ToList();

        public void CustomProcess(List<TaskData> taskDataList)
        {
            //站状态，0空，1有物料车，2满料。
            //模组型号，1   1p8s，2  1p13s，  3   ctp
            //1和2 c和d都可以进，3只能去人工位

            foreach (var item in routeConfigs)
            {
                string plcAddr = item.LocationCode.GetLocationMember("PLC工位状态");
                string plcModelAddr = item.LocationCode.GetLocationMember("PLC模组型号");
                if (string.IsNullOrEmpty(plcAddr) || string.IsNullOrEmpty(plcModelAddr))
                {
                    continue;
                }

                var tempResult = item.PlcReadWrite.ReadInt16(plcAddr);
                if(!tempResult.IsSuccess || tempResult.Content != 2)
                {
                    continue;
                }

                var tempResult2 = item.PlcReadWrite.ReadInt16(plcModelAddr);
                if (!tempResult2.IsSuccess || tempResult2.Content <= 0 || tempResult2.Content > 3)
                {
                    continue;
                }

                if (PmsTaskService.Instance.LocationHasStepTask(item.LocationCode, out string _))
                {
                    continue;
                }

                string tagetLocation = tempResult2.Content == 3 ? "上料人工" : "上料C";

                PmsTaskInfoDto data = new PmsTaskInfoDto();
                data.LocationCode = item.LocationCode;
                data.TargetLocationCode = tagetLocation;

                PmsTaskService.Instance.TaskAddNoCheck(data, new string[] { });
            }
        }
    }
}
