﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessRollerTask : ICustomProcess
    {
        private string callAddrColumnName = "PLC叫料";

        private List<RouteConfig> configList = ExcelConfig.Instance.RouteConfigList
            .Where(d => d.WareHouseCode == "上料")
            .ToList();

        public void CustomProcess(List<TaskData> taskDataList)
        {
            for (int i = 0; i < configList.Count; i++)
            {
                var config = configList[i];

                if (config.PlcReadWrite == null)
                {
                    continue;
                }

                var callAddr = config.LocationCode.GetLocationMember(callAddrColumnName);
                if (string.IsNullOrEmpty(callAddr))
                {
                    continue;
                }

                var read = config.PlcReadWrite.ReadBool(callAddr);
                if (!read.IsSuccess
                    || read.Content == false)
                {
                    continue;
                }

                var hasTask = PmsTaskService.Instance.LocationHasStepTask(config.LocationCode, out string _);
                if (hasTask)
                {
                    continue;
                }

                CLog.Instance.TaskLog.WriteInfo($"{config.LocationCode} {callAddrColumnName} {callAddr}={read.Content}");

                PmsTaskInfoDto data = new PmsTaskInfoDto();
                data.WareHouseCode = config.WareHouseCode;
                data.LocationCode = config.LocationCode;
                data.TargetWareHouseCode = "下料";
                data.TargetLocationCode = "下料点";

                PmsTaskService.Instance.TaskAddNoCheck(data, new string[] { });
            }

        }
    }
}
