﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using System;
using System.IO;
using System.Xml;
using TIBCO.Rendezvous;

namespace ProjectManagementSystem.Common.Tibco
{
    /// <summary>
    /// TIBCO初始化及监听事件
    /// </summary>
    public class TibcoConnect
    {
        private static TibcoConnect m_instance;

        private Dispatcher dispatcher = null;
        private Queue queue = null;
        public static Transport MyTransport;
        private Listener ListenerForMes;
        private bool isRun = false;
        public delegate void EventHandlerForTIBMessageReceived(string subjectName, object listener, string message);
        private string service = "";
        private string network = "";
        private string daemon = "";
        private string ICS_MES_SUB = "";
        private string MES_ICS_SUB = "";
        private string fieldName = "";
        public static TibcoConnect Instance
        {
            get
            {
                if (m_instance == null)
                {
                    m_instance = new TibcoConnect();
                    if (!m_instance.Initialize())
                    {
                        m_instance = null;
                    }
                }
                return m_instance;
            }
        }

        public bool Initialize()
        {
            try
            {
                var tibcoConfig = ExcelConfig.Instance.TibcoConfig;
                service = tibcoConfig.Service;
                network = tibcoConfig.NetWork;
                daemon = tibcoConfig.Daemon;
                ICS_MES_SUB = tibcoConfig.SubjectNameICS_MES;
                MES_ICS_SUB = tibcoConfig.SubjectNameMES_ICS;
                fieldName = tibcoConfig.FieldName;

                if (isRun)
                {
                    Terminate();
                }
                openTibEnvironment();
                createTransport();
                createQueue();
                createDispatcher();
                createOwnListener();
                isRun = true;
                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }

            return false;
        }
        // Tibrv Terminate.
        public void Terminate()
        {
            if (!isRun) return;

            try
            {
                destroyQueue();
                destroyListener();
                destroyTransfort();
            }
            catch (Exception ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
            finally
            {
                closeTibEnvironment();
                isRun = false;
            }
        }

        // Open Tibrv Environment.
        private void openTibEnvironment()
        {
            try
            {
                TIBCO.Rendezvous.Environment.Open();
            }
            catch (RendezvousException ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
        }
        private void createTransport()
        {
            try
            {
                if (daemon == "null")
                {
                    MyTransport = new NetTransport(service, network, null);
                }
                else
                {
                    MyTransport = new NetTransport(service, network, daemon);
                }
            }
            catch (RendezvousException ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
        }

        // Create Queue.
        private void createQueue()
        {
            try
            {
                queue = new Queue();
            }
            catch (RendezvousException ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
        }

        // Create Dispatcher.
        private void createDispatcher()
        {
            try
            {
                dispatcher = new Dispatcher(queue);
            }
            catch (RendezvousException ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
        }
        public void createOwnListener()
        {
            try
            {
                ListenerForMes = new Listener(queue, MyTransport, MES_ICS_SUB, null);
                ListenerForMes.MessageReceived += new MessageReceivedEventHandler(OnMessageReceivedFromMES);
            }
            catch (RendezvousException ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
        }

        // Destoroy Queue.
        private void destroyQueue()
        {
            queue.Destroy();
        }

        // Destroy Listener.
        public void destroyListener()
        {
            ListenerForMes.Destroy();
            //ListenerForICS.Destroy();
        }

        // Destroy Transport.
        private void destroyTransfort()
        {
            MyTransport.Destroy();
        }


        // Tibrv Environment Close.
        private void closeTibEnvironment()
        {
            try
            {
                GC.KeepAlive(ListenerForMes);
                TIBCO.Rendezvous.Environment.Close();
            }
            catch (Exception ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
        }
        private Message createTibMessage(string aMsg)
        {
            Message message = new Message();

            XmlDocument xmlData = new XmlDocument();
            xmlData.LoadXml(aMsg);
            message.AddField("DATA", aMsg);
            return message;
        }

        private void OnMessageReceivedFromMES(object listener, MessageReceivedEventArgs messageReceivedEventArgs)
        {
            try
            {
                Message message = messageReceivedEventArgs.Message;
                string receiveXMLString = message.GetField(fieldName).Value.ToString();

                CLog.Instance.TibcoLog.WriteDebug($"收到: {receiveXMLString}");
                string replysubjectname = "";
                string messagename = "";
                string responseMsg = TibcoReply.Instance.GetMESMessage(receiveXMLString, out replysubjectname, out messagename);
                if (messagename == "GetICSStateRequest")
                {
                    Send(responseMsg, replysubjectname);
                }

            }
            catch (Exception ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
            }
        }

        public void Send(string sendMessage, string subject)
        {
            Message message = new Message();
            message.SendSubject = subject;
            try
            {
                message.AddField(fieldName, sendMessage, 0);
                CLog.Instance.TibcoLog.WriteDebug($"发送: {sendMessage}");
                MyTransport.Send(message);
            }
            catch (RendezvousException ex)
            {
                CLog.Instance.TibcoLog.WriteExceptionCaller(ex);
                this.createTransport();
                if (MyTransport == null)
                {
                    throw new Exception("No alive server (TIB daemon, Nanotrack servers) which UI can connect.");
                }
            }
        }
    }
}
