﻿using Newtonsoft.Json;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.WebApi
{
    public class AgvsApi
    {
        private static string PublicPemkey = "-----BEGIN PUBLIC KEY-----\nMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQC55vjYCkfiaaUwQOMb0xnToVJU\n1/MKe/9oXdsoPFAc5eXSzeOtCHJFbz9aHbOvLukgwTtRXo2AQa3syvSx0zEnm+x2\nakegrfT/+9TF9pEVqPFSfhO+JteYGTGwF2KyKiE9XfWCJ+TXMfIEqHWuaA5iHDyX\naWzSldkMZOC1jLQrxwIDAQAB\n-----END PUBLIC KEY-----";
        
        private static string loginToken;
        private static DateTime loginTime;
        private static RestApi<WebMgrResponse> restApi = new RestApi<WebMgrResponse>();

        static AgvsApi()
        {
            Login();
        }

        /// <summary>
        /// Get请求
        /// </summary>
        public static bool Get<T>(string routes, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                if (!routes.Contains("Login")
                    && (DateTime.Now - loginTime).TotalMinutes > 720)
                {
                    Login();
                }

                Dictionary<string, string> header = new Dictionary<string, string>();
                header.Add("Token", loginToken);

                string url = Common.Function.AppSetting.GetAgvsWebUrl(routes);
                var resRaw = restApi.GetApiInfo(url, pars, header, body, RestSharp.Method.GET);

                var res = resRaw.Data;
                string content = resRaw.Content;

                if (res.data != null)
                {
                    var webMgrResponse = JsonConvert.DeserializeObject<WebMgrResponse>(content);
                    data = JsonConvert.DeserializeObject<T>(webMgrResponse.data.ToString());
                }
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }
            return false;
        }

        /// <summary>
        /// POST请求
        /// </summary>
        public static bool Post<T>(string routes, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                if(!routes.Contains("Login")
                    && (DateTime.Now - loginTime).TotalMinutes > 720)
                {
                    Login();
                }

                Dictionary<string, string> header = new Dictionary<string, string>();
                header.Add("Token", loginToken);

                string url = Common.Function.AppSetting.GetAgvsWebUrl(routes);
                var resRaw = restApi.GetApiInfo(url, pars, header, body, RestSharp.Method.POST);
                var res = resRaw.Data;
                string content = resRaw.Content;

                CLog.Instance.SystemLog.WriteDebug($"Post {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                if (res.data != null)
                {
                    var webMgrResponse = JsonConvert.DeserializeObject<WebMgrResponse>(content);
                    data = JsonConvert.DeserializeObject<T>(webMgrResponse.data.ToString());
                }

                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            return false;
        }

        public static LoginResponseData Login()
        {
            string pwd = "casun36689";
            string encryptPwd = RSAHelper.PEMRSAEncrypt(PublicPemkey, pwd);

            var data = new { User = "casun", Password = encryptPwd };
            if (!Post("Auth/Login", null, data, out LoginResponseData loginData)) return null;

            loginToken = loginData?.token;
            loginTime = DateTime.Now;
            return loginData;
        }
        public static MapData GetMapData()
        {
            if (!Get("Map/GetMapData", null, null, out MapData MapData)) return null;
            return MapData;
        }
    }
}
