﻿using Newtonsoft.Json;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.Logger;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Extenions;
using System.Runtime.CompilerServices;
using Mapster;

namespace ProjectManagementSystem.Common.WebApi
{
    public class CrmsApi : IPmsApi
    {
        public CrmsApi()
        {
            ////映射规则
            //var config = TypeAdapterConfig.GlobalSettings.Clone().ForType<CrmsWarehouseGroupInfo, WarehouseGroupInfo>()
            //    .Map(dest => dest.WareHouseType, src => src.GroupName)
            //    .Map(dest => dest.GroupId, src => src.ID);

            TypeAdapterConfig.GlobalSettings.ForType<CrmsWarehouseGroupInfo, WarehouseGroupInfo>()
                .Map(dest => dest.WareHouseType, src => src.GroupName)
                .Map(dest => dest.GroupId, src => src.ID);
        }

        public bool Status { get; set; }
        private RestApi<WebMgrResponse> restApi = new RestApi<WebMgrResponse>();

        /// <summary>
        /// Get请求
        /// </summary>
        /// <typeparam name="T">泛型</typeparam>
        /// <param name="apiName">接口名</param>
        /// <param name="pars">Params参数，可null</param>
        /// <param name="body">Body对象，可null</param>
        /// <param name="data">返回数据对象</param>
        /// <returns>调用成功返回True（code == 20000）</returns>
        public bool Get<T>(string apiName, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                string url = Common.Function.AppSetting.GetCrmsApiUrl(apiName);
                var res = restApi.Get(url, pars, body, out string content);

                if (res.data != null)
                {
                    var webMgrResponse = JsonConvert.DeserializeObject<WebMgrResponse>(content);
                    //var dataString = webMgrResponse.data.ToString();
                    data = JsonConvert.DeserializeObject<T>(webMgrResponse.data.ToString());
                }

                if (Status != true)
                {
                    CLog.Instance.SystemLog.WriteInfo("Pms接口恢复");
                }
                Status = true;
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            if (Status != false)
            {
                CLog.Instance.SystemLog.WriteInfo("Pms接口异常");
            }
            Status = false;
            return false;
        }

        /// <summary>
        /// POST请求
        /// </summary>
        /// <typeparam name="T">泛型</typeparam>
        /// <param name="apiName">接口名</param>
        /// <param name="pars">Params参数，可null</param>
        /// <param name="body">Body对象，可null</param>
        /// <param name="data">返回数据对象</param>
        /// <returns>调用成功返回True（code == 20000）</returns>
        public bool Post<T>(string apiName, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                string url = Common.Function.AppSetting.GetCrmsApiUrl(apiName);
                var res = restApi.Post(url, pars, body, out string content);
                CLog.Instance.SystemLog.WriteDebug($"Post {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                if (res.data != null)
                {
                    var webMgrResponse = JsonConvert.DeserializeObject<WebMgrResponse>(content);
                    data = JsonConvert.DeserializeObject<T>(webMgrResponse.data.ToString());
                }

                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            return false;
        }
        public bool Put<T>(string apiName, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                string url = Common.Function.AppSetting.GetCrmsApiUrl(apiName);
                var res = restApi.Put(url, pars, body, out string content);
                CLog.Instance.SystemLog.WriteDebug($"Put {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                //if (res.data != null)
                //{
                //    var webMgrResponse = JsonConvert.DeserializeObject<WebMgrResponse>(content);
                //    //var dataString = webMgrResponse.data.ToString();
                //    data = JsonConvert.DeserializeObject<T>(webMgrResponse.data.ToString());
                //}

                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            return false;
        }

        /// <summary>
        /// Get请求（返回原始数据模型）
        /// </summary>
        /// <param name="apiName"></param>
        /// <param name="pars"></param>
        /// <param name="body"></param>
        /// <returns></returns>
        public WebMgrResponse Get(string apiName, object pars, object body)
        {
            try
            {
                string url = Common.Function.AppSetting.GetCrmsApiUrl(apiName);
                return restApi.Get(url, pars, body, out string content);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
                return new WebMgrResponse() { message = ex.Message };
            }
        }

        /// <summary>
        /// POST请求（返回原始数据模型）
        /// </summary>
        /// <param name="apiName"></param>
        /// <param name="pars"></param>
        /// <param name="body"></param>
        /// <returns></returns>
        public WebMgrResponse Post(string apiName, object pars, object body)
        {
            try
            {
                string url = Common.Function.AppSetting.GetCrmsApiUrl(apiName);
                var res = restApi.Post(url, pars, body, out string content);
                CLog.Instance.SystemLog.WriteDebug($"Post {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                return res;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
                return new WebMgrResponse() { message = ex.Message };
            }
        }

        /// <summary>
        /// 自定义PMS接口Get
        /// </summary>
        /// <param name="apiName"></param>
        /// <param name="pars"></param>
        /// <param name="body"></param>
        /// <returns></returns>
        public WebMgrResponse CustomApiGet(string apiName, object pars, object body)
        {
            try
            {
                string url = Common.Function.AppSetting.GetPmsApiCustomUrl(apiName);
                return restApi.Get(url, pars, body, out string content);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
                return new WebMgrResponse() { message = ex.Message };
            }
        }

        /// <summary>
        /// 自定义PMS接口Post
        /// </summary>
        /// <param name="apiName"></param>
        /// <param name="pars"></param>
        /// <param name="body"></param>
        /// <returns></returns>
        public WebMgrResponse CustomApiPost(string apiName, object pars, object body)
        {
            try
            {
                string url = Common.Function.AppSetting.GetPmsApiCustomUrl(apiName);
                return restApi.Post(url, pars, body, out string content);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
                return new WebMgrResponse() { message = ex.Message };
            }
        }

        #region 常用PMS4.0接口

        public List<TaskData> GetTaskList()
        {
            if (!Get("agv/getTaskList", null, null, out List<TaskData> taskInfoList)) return null;
            return taskInfoList;
        }
        public List<TaskData> GetHistoryTaskList(DateTime startTime, DateTime endTime)
        {
            var data = new { StartTime = startTime.ToString("yyyy-MM-dd HH:mm:ss"), EndTime = endTime.ToString("yyyy-MM-dd HH:mm:ss") };
            if (!Get("agv/getHistoryTaskList", data, null, out List<TaskData> taskInfoList)) return null;
            return taskInfoList;
        }

        public TaskData GetTaskData(string taskId)
        {
            if (!Get("agv/getTask", new { taskId = taskId }, null, out TaskData taskData)) return null;
            if (taskData == null) return null;

            if (!Get("agv/getStepList", new { taskId = taskId }, null, out List<StepData> stepInfoList)) return null;
            if (stepInfoList == null) return null;

            taskData.StepList = stepInfoList;
            return taskData;
        }
        public List<TaskData> GetTaskDataList()
        {
            //if (!Get("agv/getTaskList", null, null, out List<TaskData> taskDataList)) return null;
            //if (taskDataList == null) return null;

            //for (int i = 0; i < taskDataList.Count; i++)
            //{
            //    Get("agv/getStepList", new { taskId = taskDataList[i].TaskID }, null, out List<StepData> stepInfoList);
            //    taskDataList[i].StepList = stepInfoList;

            //    if (stepInfoList == null) return null;
            //}
            //return taskDataList;

            if (!Get("agv/getTaskDataList", null, null, out List<TaskData> taskDataList)) return null;
            return taskDataList;
        }

        public TaskData GetHistoryTaskData(string taskId)
        {
            if (!Get("agv/getHistoryTaskData", new { taskId = taskId }, null, out TaskData taskData)) return null;
            return taskData;
        }

        public List<StepData> GetStepList(string taskId)
        {
            if (!Get("agv/getStepList", new { taskId = taskId }, null, out List<StepData> stepInfoList)) return null;
            return stepInfoList;
        }
        public List<StepData> GetHistoryStepList(string taskId)
        {
            if (!Get("agv/getHistoryStepList", new { taskId = taskId }, null, out List<StepData> stepInfoList)) return null;
            return stepInfoList;
        }

        public List<Freight> GetFreights(int agvId)
        {
            if (!Get("agv/getFreights", new { carrierIndex = agvId }, null, out List<Freight> agvData)) return null;
            return agvData;
        }

        public ResultAgvData GetCarrier(int agvId)
        {
            if (!Get("agv/getCarrier", new { agvId = agvId }, null, out ResultAgvData agvData)) return null;

            agvData.AlarmList = agvData.AlarmDTOList;
            return agvData;
        }
        public List<ResultAgvData> GetAllCarrier()
        {
            if (!Get("agv/getAllCarrier", null, null, out ResultCarrierInfo carrierInfo)) return null;

            for (int i = 0; i < carrierInfo.resultAgvDataDTOList.Count; i++)
            {
                carrierInfo.resultAgvDataDTOList[i].AlarmList = carrierInfo.resultAgvDataDTOList[i].AlarmDTOList;
            }
            return carrierInfo.resultAgvDataDTOList;
        }

        public List<WarehouseInfo> GetWarehouseListInfo(string warehouseId)
        {
            if (!Get("agv/getWarehouseListInfo", new { warehouseId = warehouseId }, null, out List<WarehouseInfo> whouseInfoList)) return null;
            return whouseInfoList;
        }
        public List<TaskBookInfo> GetTaskBookList()
        {
            if (!Get("agv/getTaskBookList", null, null, out List<TaskBookInfo> taskBookInfoList)) return null;
            return taskBookInfoList;
        }
        public List<OperateInfo> GetOperateList(string name)
        {
            if (!Get("agv/getOperateList", new { name = name }, null, out List<OperateInfo> operateInfoList)) return null;
            return operateInfoList;
        }
        public List<WarehouseGroupInfo> GetwarehouseGroupType()
        {
            if (!Get("agv/allWarehouseGroup", null, null, out List<CrmsWarehouseGroupInfo> warehouseGroupInfoList)) return null;

            return warehouseGroupInfoList.Adapt<List<WarehouseGroupInfo>>();
        }
        public Dictionary<int, RequestPmsTaskInfo> GetPmsBehavior()
        {
            if (!Get("agv/getPmsBehavior", null, null, out Dictionary<int, RequestPmsTaskInfo> requestPmsTaskInfoDict)) return null;

            foreach (var item in requestPmsTaskInfoDict.Keys)
            {
                requestPmsTaskInfoDict[item].agvId = item;
            }
            return requestPmsTaskInfoDict;
        }

        public MapData GetMapData()
        {
            if (!Get("map/getMapData", null, null, out MapData data)) return null;
            return data;
        }

        public bool UpdateStepWarehouse(string taskId, int stepId, string warehouseId)
        {
            var data = new { taskId = taskId, stepId = stepId, warehouseId = warehouseId };
            return Post("agv/updateStepWarehouse", null, data, out string _);
        }
        public bool UpdateStepAgvBehavior(string TaskId, int StepId, bool IsAgvBehavior, int AgvBehavior, int Param1, int Param2, int Param3, int PmsBehavior)
        {
            var data = new
            {
                taskId = TaskId,
                stepId = StepId,
                isAgvBehavior = IsAgvBehavior,
                agvBehavior = AgvBehavior,
                param1 = Param1,
                param2 = Param2,
                param3 = Param3,
                pmsBehavior = PmsBehavior
            };

            return Post("agv/updateStepAgvBehavior", null, data, out string _);
        }
        public bool TaskStepSkip(string taskId, int stepId)
        {
            var data = new { taskId = taskId, stepId = stepId };
            return Post("agv/taskStepSkip", null, data, out object _);
        }
        public bool DeleteTaskStep(string taskId, int stepId)
        {
            var data = new { taskId = taskId, stepId = stepId };
            return Post("agv/deleteTaskStep", null, data, out string _);
        }
        public bool TaskAddStep(string TaskId, int Station, bool IsAgvBehavior, int AgvBehavior, int Param1, int PmsBehavior)
        {
            var data = new
            {
                taskId = TaskId,
                station = Station,
                isAgvBehavior = IsAgvBehavior,
                agvBehavior = AgvBehavior,
                param1 = Param1,
                pmsBehavior = PmsBehavior
            };

            return Post("agv/taskAddStep", null, data, out string _);
        }

        public bool AddOperate(int behavior, string remark)
        {
            var requestOperateInfo = new
            {
                name = behavior.ToString(),
                code = behavior,
                isAgvBehavior = false,
                isDelete = true,
                isEdit = true,
                remark = remark
            };

            return Post("agv/addOperate", null, requestOperateInfo, out string _);
        }
        public bool UpdateOperate(OperateInfo operateInfo)
        {
            var requestOperateInfo = new
            {
                id = operateInfo.ID,
                name = operateInfo.Name,
                code = operateInfo.Code,
                isAgvBehavior = false,
                isDelete = true,
                isEdit = true,
                remark = operateInfo.Remark
            };
            return Post("agv/updateOperate", null, requestOperateInfo, out string _);
        }
        public bool AddWarehouse(string wareHouseID, int stationID, int r1 = 0, int r2 = 0, int r3 = 0, string groupId = "")
        {
            var requestInfo = new
            {
                warehouseId = wareHouseID,
                warehouseName = "",
                stationId = stationID,
                relationStationId1 = (r1 <= 0) ? "" : r1.ToString(),
                relationStationId2 = (r2 <= 0) ? "" : r2.ToString(),
                relationStationId3 = (r3 <= 0) ? "" : r3.ToString(),
                groupId = groupId
            };

            return Post("agv/addWarehouse", null, requestInfo, out string _);
        }
        public bool UpdateWarehouse(string wareHouseID, int stationID, int r1 = 0, int r2 = 0, int r3 = 0, string groupId = "")
        {
            var requestInfo = new
            {
                warehouseId = wareHouseID,
                warehouseName = "",
                stationId = stationID,
                relationStationId1 = (r1 <= 0) ? "" : r1.ToString(),
                relationStationId2 = (r2 <= 0) ? "" : r2.ToString(),
                relationStationId3 = (r3 <= 0) ? "" : r3.ToString(),
                groupId = groupId
            };

            return Put("agv/updateWarehouse", null, requestInfo, out object _);
        }

        public bool CarrierPause(int agvId)
        {
            return Get("agv/carrierPause", new { carrierIndex = agvId }, null, out string _);
        }
        public bool CarrierResume(int agvId)
        {
            return Get("agv/carrierResume", new { carrierIndex = agvId }, null, out string _);
        }
        public bool SetCarrierLogicBits(int agvId, int bit, int value)
        {
            var data = new { agvId = agvId, bit = bit, value = value };
            return Post("agv/setCarrierLogicBits", null, data, out int[] _);
        }
        public bool SetCarrierLogicValues(int agvId, int bit, int[] values)
        {
            var data = new { agvId = agvId, bit = bit, values = values };
            return Post("agv/setCarrierLogicValues", null, data, out int[] _);
        }
        public bool SetTaskPriority(string taskId, int priority)
        {
            var data = new { taskId = taskId, priority = priority };
            return Post("agv/setTaskPriority", null, data, out string _);
        }
        public bool SendTransparentData(int agvId, byte[] data)
        {
            var transData = new
            {
                agvId = agvId,
                data = data,
                len = data.Length
            };

            return Post("agv/sendTransparentData", null, transData, out string _);
        }
        public bool SetPmsBehaviorFinish(string taskId, int agvId, int pmsBehavior)
        {
            var data = new
            {
                agvId = agvId,
                taskId = taskId,
                pmsBehavior = pmsBehavior,
            };

            return Post("agv/setPmsBehaviorFinish", null, data, out object _);
        }

        public WebMgrResponse TaskAddResponse(MidTaskInfoEx data)
        {
            data.SerializeParameters();
            var midTask = new
            {
                taskId = data.TaskId,
                templateName = data.TemplateName,
                parameters = data.Parameters,
                priority = data.Priority,
                agvId = data.AgvId,
                agvType = data.AgvType,
                parentTaskId = data.ParentTaskId
            };

            return Post("agv/taskAdd", null, midTask);
        }
        public bool TaskAdd(MidTaskInfoEx data, [CallerMemberName] string logKeyWord = null)
        {
            data.SerializeParameters();
            var midTask = new
            {
                taskId = data.TaskId,
                templateName = data.TemplateName,
                parameters = data.Parameters,
                priority = data.Priority,
                agvId = data.AgvId,
                agvType = data.AgvType,
                parentTaskId = data.ParentTaskId
            };

            bool result = Post("agv/taskAdd", null, midTask, out MidTaskInfo _);
            CLog.Instance.GetAgvLog(data.AgvId).WriteInfo(logKeyWord, $"{data.TaskId} {data.ParentTaskId} AGV{data.AgvId} AgvType{data.AgvType} {data.TemplateName} {data.Parameters} 任务添加{result.ToChineseString()}");
            return result;
        }
        public bool TaskCancel(string taskId)
        {
            var data = new { taskId = taskId, isCancelRely = true };
            return Post("agv/taskCancel", null, data, out object _);
        }
        public bool TaskEnd(string taskId)
        {
            var data = new { taskId = taskId, isReleaseAGV = false };
            return Post("agv/taskEnd", null, data, out object _);
        }
        public bool TaskPause(string taskId)
        {
            return Post("agv/taskPause", null, taskId, out string _);
        }
        public bool TaskResume(string taskId)
        {
            return Post("agv/taskResume", null, taskId, out string _);
        }
        public bool TaskChangeRely(string taskId, string parentTaskId)
        {
            var data = new { taskId = taskId, parentTaskId = parentTaskId };
            bool result = Post("agv/taskChangeRely", null, data, out object _);
            //CLog.Instance.TaskLog.WriteInfo($"{taskId} 更改父任务{result.ToChineseString()} {parentTaskId}");
            return result;
        }
        public bool TaskChangeAssignAGV(string taskId, int agvId)
        {
            var data = new { taskId = taskId, agvId = agvId };
            return Post("agv/taskChangeAssignAGV", null, data, out string _);
        }
        public bool TaskStopReceipt()
        {
            return Post("agv/taskStopReceipt", null, null, out string _);
        }
        public bool TaskContinueReceipt()
        {
            return Post("agv/taskContinueReceipt", null, null, out string _);
        }

        #endregion
    }
}
