﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using Newtonsoft.Json;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.WebApi
{
    public class CustomerApi
    {
        private static string url = AppSetting.TryGetValue<string>("CustomerAddress");
        private static RestApi<ResponseResult> restApi = new RestApi<ResponseResult>();

        public static bool ReportAgvARrrived(string taskId, string station, string type, out string content)
        {
            string apiName = "ReportAgvARrrived";
            var p = new { TaskId = taskId, Pos = station};

            var result = CallWebService<bool>(apiName, p, d => d, out content);

            return result;
        }

        public static bool AgvNoReturn(string agvNo, string MESNo, out string content)
        {
            string apiName = "AgvNoReturn";
            var body = new { agvNo = agvNo, MESNo = MESNo };
            var result = WmsApi(apiName, body, out content);

            return result;
        }

        public static bool AgvArrive(string bin, string agvNo, string MESNo, out string content)
        {
            string apiName = "AgvArrive";
            var body = new { bin = bin, agvNo = agvNo, MESNo = MESNo };
            var result = WmsApi(apiName, body, out content);

            return result;
        }

        public static bool AgvLeave(string bin, string agvNo, string MESNo, out string content)
        {
            string apiName = "AgvLeave";
            var body = new { bin = bin, agvNo = agvNo, MESNo = MESNo };
            var result = WmsApi(apiName, body, out content);

            return result;
        }

        public static bool AgvAlarm(string msg, string agvNo, string MESNo, out string content)
        {
            string apiName = "AgvAlarm";
            var body = new { msg = msg, agvNo = agvNo, MESNo = MESNo };
            var result = WmsApi(apiName, body, out content);

            return result;
        }

        private static string GetApiUrl(string apiName)
        {
            string apiUrl = AppSetting.TryGetValue<string>($"{apiName}Url");
            if (string.IsNullOrEmpty(apiUrl))
            {
                apiUrl = $@"{url}{apiName}";
            }

            return apiUrl;
        }

        private static bool WmsApi(string apiName, object body, out string content)
        {
            content = null;
            bool result = false;

            try
            {
                string apiUrl = GetApiUrl(apiName);

                var res = restApi.Post(apiUrl, null, body, out content);
                CLog.Instance.SystemLog.WriteDebug($"Post {apiUrl}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                result = res.code == 200 && res.success;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);

                content = ex.Message;
            }

            return result;
        }

        private static bool CallWebService<T>(string apiName, object pars, Predicate<T> predicate, out string content)
        {
            content = null;
            bool result = false;

            try
            {
                string apiUrl = GetApiUrl(apiName);
                var webServiceApi = new RestApi<T>();

                var res = webServiceApi.Get(apiUrl, pars, null, out content);
                CLog.Instance.SystemLog.WriteDebug($"Get {apiUrl}{Environment.NewLine}  {JsonConvert.SerializeObject(pars)}{Environment.NewLine}  {content}");

                var data = XmlTools.XmlDeserialize<T>(content);
                result = predicate(data);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);

                content = ex.Message;
            }

            return result;
        }
    }



    public class CustomerMesApi
    {
        private static string url = AppSetting.TryGetValue<string>("CustomerAddress");
        private static RestApi<ResponseResult2> restApi2 = new RestApi<ResponseResult2>();

        private static string GetApiUrl(string apiName)
        {
            string apiUrl = AppSetting.TryGetValue<string>($"{apiName}Url");
            if (string.IsNullOrEmpty(apiUrl))
            {
                apiUrl = $@"{url}{apiName}";
            }

            return apiUrl;
        }

        public static bool AgvChargePileData(object body, out string content)
        {
            string apiName = "AgvChargePileData";
            var result = MesApi(apiName, body, out content);

            return result;
        }

        public static bool AgvAlarmData(object body, out string content)
        {
            string apiName = "AgvAlarmData";
            var result = MesApi(apiName, body, out content);

            return result;
        }

        public static bool AgvStatusData(object body, out string content)
        {
            string apiName = "AgvStatusData";
            var result = MesApi(apiName, body, out content);

            return result;
        }

        private static bool MesApi(string apiName, object body, out string content)
        {
            content = null;
            bool result = false;

            try
            {
                string apiUrl = GetApiUrl(apiName);

                var res = restApi2.Post(apiUrl, null, body, out content);
                result = res.resultCode == "200";

                if (!result)
                {
                    CLog.Instance.SystemLog.WriteDebug($"Post {apiUrl}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");
                }
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);

                content = ex.Message;
            }

            return result;
        }
    }
}
