﻿using Newtonsoft.Json;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.WebApi
{
    public class DeviceApi
    {
        private static RestApi<WebMgrResponse> restApi = new RestApi<WebMgrResponse>();

        /// <summary>
        /// Get请求
        /// </summary>
        public static bool Get<T>(string routes, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                string url = Common.Function.AppSetting.GetAgvsWebUrl(routes);
                var resRaw = restApi.GetApiInfo(url, pars, null, body, RestSharp.Method.GET);

                var res = resRaw.Data;
                string content = resRaw.Content;

                if (res.data != null)
                {
                    var webMgrResponse = JsonConvert.DeserializeObject<WebMgrResponse>(content);
                    data = JsonConvert.DeserializeObject<T>(webMgrResponse.data.ToString());
                }
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }
            return false;
        }

        public static List<PileV4VO> GetAllPile()
        {
            if (!Get("PileV4/GetAllPile", null, null, out List<PileV4VO> dataList)) return null;
            return dataList;
        }
    }

    public class PileV4VO
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public bool OnLine { get; set; }
        public string IPv4 { get; set; }
        public double Voltage { get; set; }
        public double Current { get; set; }
        public double Temperature { get; set; }
        public bool Occupied { get; set; }
        public int Carrier { get; set; }
        public ChargingStatus ChargingStatus { get; set; }
        public AlarmStatus AlarmStatus { get; set; }
        public int Station { get; set; }
    }
    public enum ChargingStatus { ManulCharging, ManulFree, AutoCharging, AutoFree, AutoCompleted, Emergency }
    public enum AlarmStatus { None, OverCurrent, OverVoltage, OverTemperature, AgvUndetected, Other }
}
