﻿using ProjectManagementSystem.Common.Logger;
using Swashbuckle.Application;
using System;
using System.Security.Principal;
using System.Web.Http;
using System.Web.Http.Cors;
using System.Web.Http.SelfHost;

namespace ProjectManagementSystem.WebApi.Service
{
    public class WebApiServerHost
    {
        private HttpSelfHostServer server;

        public WebApiServerHost()
        {
        }

        public bool IsAdministrator()
        {
            WindowsIdentity current = WindowsIdentity.GetCurrent();
            WindowsPrincipal windowsPrincipal = new WindowsPrincipal(current);
            return windowsPrincipal.IsInRole(WindowsBuiltInRole.Administrator);
        }

        public bool StartServer()
        {
            try
            {
                if (!IsAdministrator()) throw new Exception("需要管理员权限启动程序");

                var config = new HttpSelfHostConfiguration("http://localhost:63300");
                config.Formatters.Remove(config.Formatters.XmlFormatter);

                config.Routes.MapHttpRoute(
                    name: "DefaultApi",
                    routeTemplate: "api/{controller}/{action}/{id}",
                    defaults: new { id = RouteParameter.Optional }
                );

                config.EnableCors(new EnableCorsAttribute("*", "*", "*"));

                //启用Swagger
                //config.EnableSwagger(c =>
                //     {
                //         c.SingleApiVersion("v2", "PMS定制接口");

                //         //var baseDirectory = AppDomain.CurrentDomain.BaseDirectory;
                //         //var commentsFileName = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name + ".XML";
                //         //var commentsFile = System.IO.Path.Combine(baseDirectory, commentsFileName);
                //         //c.IncludeXmlComments(commentsFile);

                //         //c.RootUrl(req =>
                //         //    req.RequestUri.GetLeftPart(UriPartial.Authority) +
                //         //    req.GetRequestContext().VirtualPathRoot.TrimEnd('/'));
                //     })
                //     .EnableSwaggerUi(c =>
                //     {
                //         c.DocExpansion(DocExpansion.List);
                //     });

                //添加Filters
                config.Filters.Add(new Filters.GlobalExceptionFilterAttribute());
                //config.Filters.Add(new Filters.GlobalActionFilterAttribute());

                server = new HttpSelfHostServer(config);
                server.OpenAsync().Wait();

                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("WebApi", ex);
            }

            return false;
        }
    }
}
