﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectManagementSystem.UI
{
    public partial class DataGridUI : UserControl, IDisplayUI
    {
        private int selectRowIndexOfordersDataGridView;         // 记录选中的行号
        private int selectColumnIndexOfordersDataGridView;      // 记录选中的列号
        private int scrollingRowIndexOfordersDataGridView;      // 记录垂直滚动条位置
        private int scrollingColumnIndexOfordersDataGridView;   // 记录水平滚动条的位置

        public DataGridUI()
        {
            InitializeComponent();
        }


        public virtual bool Init()
        {
            return true;
        }

        public virtual void UpdateDisplay()
        {
        }

        public DataTable GetDataTable()
        {
            DataTable dt = new DataTable();
            for (int count = 0; count < dataGridView1.Columns.Count; count++)
            {
                if (dataGridView1.Columns[count].Visible)
                {
                    DataColumn dc = new DataColumn(dataGridView1.Columns[count].HeaderText.ToString());
                    dt.Columns.Add(dc);
                }
            }


            for (int count = 0; count < dataGridView1.Rows.Count; count++)
            {
                DataRow dr = dt.NewRow();

                int colCount = 0;
                for (int countsub = 0; countsub < dataGridView1.Columns.Count; countsub++)
                {
                    if (dataGridView1.Columns[countsub].Visible)
                    {
                        dr[colCount] = dataGridView1.Rows[count].Cells[countsub].Value.ToString();
                        colCount++;
                    }
                }
                dt.Rows.Add(dr);
            }

            return dt;
        }

        public string GetCurrentColValue(string colName)
        {
            if (dataGridView1.CurrentRow == null)
            {
                return null;
            }

            if (!dataGridView1.Columns.Contains(colName))
            {
                return null;
            }

            int DGVCurrentRow = dataGridView1.CurrentRow.Index;


            if (DGVCurrentRow < 0 || dataGridView1.Rows[DGVCurrentRow].Cells[colName].Value == null)
            {
                return null;
            }

            string strOrderID = dataGridView1.Rows[DGVCurrentRow].Cells[colName].Value.ToString();
            return strOrderID;
        }

        public void ShowDataGrid<T>(List<T> dataSource) where T : class
        {
            this.Invoke(new MethodInvoker(() =>
            {
                this.SuspendLayout();
                if (dataGridView1.CurrentCell != null && dataGridView1.RowCount != 0 && dataGridView1.ColumnCount != 0)
                {
                    selectRowIndexOfordersDataGridView = dataGridView1.CurrentCell.RowIndex;
                    selectColumnIndexOfordersDataGridView = dataGridView1.CurrentCell.ColumnIndex;
                    scrollingRowIndexOfordersDataGridView = dataGridView1.FirstDisplayedScrollingRowIndex;
                    scrollingColumnIndexOfordersDataGridView = dataGridView1.FirstDisplayedScrollingColumnIndex;
                }

                this.dataGridView1.DataSource = dataSource;

                int n = dataGridView1.RowCount;
                int m = dataGridView1.ColumnCount;

                if (n != 0 && m != 0)
                {
                    if (selectRowIndexOfordersDataGridView < n && selectColumnIndexOfordersDataGridView < m &&
                        scrollingRowIndexOfordersDataGridView < n && scrollingColumnIndexOfordersDataGridView < m &&
                        selectRowIndexOfordersDataGridView >= 0 && selectColumnIndexOfordersDataGridView >= 0 &&
                        scrollingRowIndexOfordersDataGridView >= 0 && scrollingColumnIndexOfordersDataGridView >= 0
                        && dataGridView1.Rows[selectRowIndexOfordersDataGridView].Cells[selectColumnIndexOfordersDataGridView].Visible)
                    {
                        dataGridView1.CurrentCell = dataGridView1.Rows[selectRowIndexOfordersDataGridView].Cells[selectColumnIndexOfordersDataGridView];
                        dataGridView1.FirstDisplayedScrollingRowIndex = scrollingRowIndexOfordersDataGridView;
                        dataGridView1.FirstDisplayedScrollingColumnIndex = scrollingColumnIndexOfordersDataGridView;
                    }
                }

                foreach (DataGridViewColumn column in dataGridView1.Columns)
                {
                    if (column.Name.Contains("Time"))
                    {
                        column.DefaultCellStyle.Format = "G";
                    }
                    if (column.Name.Contains("HEX"))
                    {
                        column.DefaultCellStyle.Format = "X2";
                    }
                }

                var keyInfo = Common.Config.ExcelConfig.Instance.ColumnKeyInfoDict;
                foreach (DataGridViewColumn column in dataGridView1.Columns)
                {
                    if (keyInfo.ContainsKey(column.Name))
                    {
                        column.HeaderText = keyInfo[column.Name].HeaderText;
                        column.Visible = keyInfo[column.Name].Visible;

                        if (!string.IsNullOrEmpty(keyInfo[column.Name].Format))
                        {
                            column.DefaultCellStyle.Format = keyInfo[column.Name].Format;
                        }
                    }
                    else
                    {
                        var displayName = typeof(T)?.GetProperty(column.Name)?.GetCustomAttribute<DisplayNameAttribute>()?.DisplayName;

                        if (!string.IsNullOrEmpty(displayName))
                        {
                            column.HeaderText = displayName;
                        }
                        //else
                        //{
                        //    column.Visible = false;
                        //}
                    }
                }

                this.ResumeLayout(false);
            }));
        }
    }
}
