﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Threading;
using DbCommon.Enties.DbModels;
using DbCommon.BusinessCore.BaseCore;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Logger;
using ScottPlot;
using ProjectManagementSystem.Common.Extenions;

namespace ProjectManagementSystem.UI
{
    public partial class WmsPlotUI : UserControl, IDisplayUI
    {
        private FormsPlot formsPlot1 = new FormsPlot();
        private FormsPlot formsPlot2 = new FormsPlot();
        private Plot plotStatus = new Plot();
        private Plot plotMaterial = new Plot();

        private double[] statusCounts;
        private string[] statusLables;
        private Color[] statusColors;

        private double[] materialCounts;
        private string[] materialLables;

        private string m_wareHouseCode = "库位";

        private LocationPropertyManager locationManager;

        public WmsPlotUI()
        {
            InitializeComponent();
        }

        private void WmsPlotUI_Load(object sender, EventArgs e)
        {
        }

        public bool Init()
        {
            this.DoubleBuffered = true;//设置本窗体
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true); // 禁止擦除背景.
            SetStyle(ControlStyles.DoubleBuffer, true); // 双缓冲

            locationManager = new LocationPropertyManager();

            PlotStatusInit();
            PlotMaterialInit();

            return true;
        }

        private void PlotStatusInit()
        {
            formsPlot1.Dock = DockStyle.Fill;
            splitContainer2.Panel1.Controls.Add(formsPlot1);

            formsPlot1.Reset(plotStatus);
            formsPlot1.Refresh();
        }

        private void PlotMaterialInit()
        {
            formsPlot2.Dock = DockStyle.Fill;
            splitContainer2.Panel2.Controls.Add(formsPlot2);

            formsPlot2.Reset(plotMaterial);
            formsPlot2.Refresh();
        }

        public void UpdateDisplay()
        {
            if (!this.Visible)
            {
                return;
            }

            var lists = locationManager.QueryDataList(m_wareHouseCode);

            PlotStatus(lists);
            PlotMaterial(lists);
        }


        public void PlotStatus(List<LocationProperty> locationList)
        {
            bool reset = true;
            //统计总状态
            LocationStatus[] locationStatuses = locationList.GroupBy(d => d.Status).Select(d => d.Key).ToArray();
            if (statusCounts == null
                || statusCounts.Length != locationStatuses.Length)
            {
                statusCounts = locationStatuses.Select(d => Convert.ToDouble(locationList.Count(f => f.Status == d))).ToArray();
                statusColors = locationStatuses.Select(d => d.ToColor()).ToArray();
                var statusLablesTemp = locationStatuses.Select(d => d.ToStatusString()).ToArray();

                statusLables = Enumerable.Range(0, statusCounts.Length)
                       .Select(i => $"{statusLablesTemp[i]}({statusCounts[i]})")
                       .ToArray();

                reset = true;
            }
            else
            {
                for (int i = 0; i < locationStatuses.Length; i++)
                {
                    var status = locationStatuses[i];

                    statusCounts[i] = locationList.Count(f => f.Status == status);
                    statusColors[i] = status.ToColor();
                    statusLables[i] = $"{status.ToStatusString()}({statusCounts[i]})";
                }
                reset = false;
            }

            if (statusLables.Length == 0
                || statusCounts.Length == 0)
            {
                plotStatus.Clear();
                formsPlot1.Reset();
            }
            else
            {
                if (reset)
                {
                    plotStatus.Clear();
                    var pie1 = plotStatus.AddPie(statusCounts);
                    pie1.SliceLabels = statusLables;
                    pie1.SliceFillColors = statusColors;
                    //pie1.ShowValues = true;
                    //pie1.ShowPercentages = true;
                    pie1.ShowLabels = true;
                    plotStatus.Legend();
                }
            }

            formsPlot1.Refresh();
        }

        public void PlotMaterial(List<LocationProperty> locationList)
        {
            bool reset = true;
            //统计总物料
            string[] materialLablesTemp = locationList
                .Where(d => d.Status != LocationStatus.Empty && d.Status != LocationStatus.None)
                .GroupBy(d => d.MaterialId)
                .Select(d => d.Key)
                .ToArray();

            if (materialLables == null
                || materialLables.Length != materialLablesTemp.Length)
            {
                materialLables = materialLablesTemp;
                materialCounts = materialLablesTemp
                    .Select(d => Convert.ToDouble(locationList
                                .Where(f => f.MaterialId == d)
                                .Sum(f => { bool ret = int.TryParse(f.MaterialNumber, out int number); return ret ? number : 1; })))
                    .ToArray();
                reset = true;
            }
            else
            {
                reset = false;
                for (int i = 0; i < materialLablesTemp.Length; i++)
                {
                    var material = materialLablesTemp[i];
                    int count = locationList
                        .Where(f => f.MaterialId == material)
                        .Sum(f => { bool ret = int.TryParse(f.MaterialNumber, out int number); return ret ? number : 1; });

                    if (materialLables[i] != material
                        || materialCounts[i] != count)
                    {
                        reset = true;
                    }

                    materialLables[i] = material;
                    materialCounts[i] = count;
                }
            }

            if (materialLables.Length == 0
                || materialCounts.Length == 0)
            {
                plotMaterial.Clear();
                formsPlot2.Reset();
            }
            else
            {
                if (reset)
                {
                    plotMaterial.Clear();
                    var bar = plotMaterial.AddBar(materialCounts);
                    bar.ShowValuesAboveBars = true;

                    //plotMaterial.XLabel("物料");
                    plotMaterial.YLabel("数量");
                    plotMaterial.XTicks(materialLables);
                    plotMaterial.SetAxisLimits(yMin: 0);

                    formsPlot2.Reset(plotMaterial);
                }
            }

            formsPlot2.Refresh();
        }
    }
}
