﻿using Microsoft.Xaml.Behaviors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace ProjectManagementSystemView.Infrastructure.Behaviors
{
    public class MouseClickSelectedItem3dBehavior : Behavior<FrameworkElement>
    {
        public object SelectedItem
        {
            get { return (object)GetValue(SelectedItemProperty); }
            set { SetValue(SelectedItemProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedItem.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SelectedItemProperty =
            DependencyProperty.Register("SelectedItem", typeof(object), typeof(MouseClickSelectedItem3dBehavior));


        private void OnMouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (e.ChangedButton == System.Windows.Input.MouseButton.Left && e.Source is HelixToolkit.Wpf.HelixViewport3D viewport3D)
            {
                var element = viewport3D.FindNearestVisual(e.GetPosition(viewport3D));
                if (element is Views.BoxLocationVisual3D element1 && element1.LocationData is object v)
                {
                    SelectedItem = v;
                }
                else
                {
                    SelectedItem = null;
                }
            }
        }

        protected override void OnAttached()
        {
            AssociatedObject.MouseUp += OnMouseUp;
        }

        protected override void OnDetaching()
        {
            AssociatedObject.MouseUp -= OnMouseUp;
        }
    }
}
