﻿using Mapster;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace ProjectManagementSystemView.Infrastructure.Extensions
{
    public static class ObservableCollectionExtensions
    {
        public static void Update<TS, TV>(this ObservableCollection<TV> VmDatas,
            IEnumerable<TS> dataList,
            Func<TS, TV, bool> predicate,
            Action<TS, TV> action = null)
        {
            if (dataList == null || dataList.Count() == 0)
            {
                if (VmDatas.Count > 0)
                {
                    Application.Current.Dispatcher.Invoke(() =>
                    {
                        VmDatas.Clear();
                    });
                }
                return;
            }


            for (int i = VmDatas.Count - 1; i >= 0; i--)
            {
                var vmData = VmDatas[i];

                var data = dataList.FirstOrDefault(s => predicate(s, vmData));
                if (data == null)
                {
                    Application.Current.Dispatcher.Invoke(() =>
                    {
                        VmDatas.Remove(vmData);
                    });
                }
            }

            foreach (var s in dataList)
            {
                var vmData = VmDatas.FirstOrDefault(v => predicate(s, v));
                if (vmData == null)
                {
                    vmData = s.Adapt<TV>();
                    action?.Invoke(s, vmData);

                    Application.Current.Dispatcher.Invoke(() =>
                    {
                        VmDatas.Add(vmData);
                    });
                }
                else
                {
                    s.Adapt(vmData);
                    action?.Invoke(s, vmData);
                }
            }
        }
    }
}
