﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using Prism.Commands;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Common.WebApi;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace ProjectManagementSystemView.ViewModels
{
    [AddINotifyPropertyChangedInterface]
    public class DebugViewModel
    {
        private LocationPropertyManager locationManager = new LocationPropertyManager();
        private WareHouseManager wareHouseManager = new WareHouseManager();

        public string WareHouseCode1 { get; set; }
        public string WareHouseCode2 { get; set; }
        public string LocationCode1 { get; set; }
        public string LocationCode2 { get; set; }

        public string P1 { get; set; }
        public string P2 { get; set; }

        public ObservableCollection<WareHouse> WareHouseCollection { get; set; }
        public List<LocationProperty> LocationPropertyCollection1 { get; set; }
        public List<LocationProperty> LocationPropertyCollection2 { get; set; }

        public DelegateCommand LoadedCommand { get; private set; }
        public DelegateCommand DropDownOpened1 { get; private set; }
        public DelegateCommand DropDownOpened2 { get; private set; }
        public DelegateCommand CallTaskCommand { get; private set; }

        public DelegateCommand TestCommand { get; private set; }

        public DebugViewModel()
        {
            WareHouseCollection = new ObservableCollection<WareHouse>();
            LocationPropertyCollection1 = new List<LocationProperty>();
            LocationPropertyCollection2 = new List<LocationProperty>();

            LoadedCommand = new DelegateCommand(() =>
            {
                if (string.IsNullOrEmpty(LocationCode1)
                && string.IsNullOrEmpty(LocationCode2))
                {
                    WareHouseCollection.Clear();
                    var list = wareHouseManager.GetList();
                    list.ForEach(p => WareHouseCollection.Add(p));
                }
            });

            DropDownOpened1 = new DelegateCommand(() =>
            {
                string lastLocation = LocationCode1;
                LocationPropertyCollection1 = locationManager.GetList(d => d.WareHouseCode == WareHouseCode1);
                LocationCode1 = lastLocation;
            });

            DropDownOpened2 = new DelegateCommand(() =>
            {
                string lastLocation = LocationCode2;
                LocationPropertyCollection2 = locationManager.GetList(d => d.WareHouseCode == WareHouseCode2);
                LocationCode2 = lastLocation;
            });

            CallTaskCommand = new DelegateCommand(CallTask);
            TestCommand = new DelegateCommand(Test);
        }

        private void Test()
        {
            try
            {
                var result = CustomerApi.ReportAgvARrrived(P1, P2, null, out string content);
                ShowMsg($"测试{result.ToChineseString()} {content}");
            }
            catch (Exception ex)
            {
                ShowErrMsg($"测试发生异常：{ex.Message}");
            }
        }

        private void CallTask()
        {
            if (string.IsNullOrEmpty(WareHouseCode1)
                && string.IsNullOrEmpty(LocationCode1))
            {
                ShowMsg("请选择起点位置！");
                return;
            }
            if (string.IsNullOrEmpty(WareHouseCode2)
                && string.IsNullOrEmpty(LocationCode2))
            {
                ShowMsg("请选择终点位置！");
                return;
            }

            string text = $"确定要添加此任务？{Environment.NewLine}{WareHouseCode1}({LocationCode1})->{WareHouseCode2}({LocationCode2})";
            var dialogResult = MessageBox.Show(text, "提示", MessageBoxButton.OKCancel, MessageBoxImage.Question, MessageBoxResult.Cancel);
            if (dialogResult != MessageBoxResult.OK)
            {
                return;
            }

            try
            {
                PmsTaskInfo data = new PmsTaskInfo();
                data.WareHouseCode = WareHouseCode1;
                data.LocationCode = LocationCode1;
                data.TargetWareHouseCode = WareHouseCode2;
                data.TargetLocationCode = LocationCode2;

                var result = Crms.PmsApi.CustomApiPost("TaskAdd", null, data);
                if (result?.code == 20000)
                {
                    ShowMsg($"添加任务成功，{result?.message}");
                }
                else
                {
                    ShowErrMsg($"添加任务失败，{result?.message}");
                }
            }
            catch (Exception ex)
            {
                ShowErrMsg($"添加任务发生异常：{ex.Message}");
            }
        }

        private void ShowMsg(string text)
        {
            MessageBox.Show(text, "提示", MessageBoxButton.OK, MessageBoxImage.Information);
        }
        private void ShowErrMsg(string text)
        {
            MessageBox.Show(text, "提示", MessageBoxButton.OK, MessageBoxImage.Error);
        }
    }
}
