﻿using DbCommon.BusinessCore.BaseCore;
using Mapster;
using MaterialDesignThemes.Wpf;
using Prism.Commands;
using Prism.Services.Dialogs;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystemView.Infrastructure;
using ProjectManagementSystemView.Infrastructure.Models;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystemView.ViewModels.Dialogs
{
    [AddINotifyPropertyChangedInterface]
    public class SimpleTaskViewModel : IDialogHostAware
    {
        private LocationPropertyManager locationManager = new LocationPropertyManager();
        public ObservableCollection<string> WareHouseCodeCollection1 { get; set; }
        public ObservableCollection<string> LocationCodeCollection1 { get; set; }
        public ObservableCollection<string> MaterialIdCollection { get; set; }

        public ObservableCollection<string> WareHouseCodeCollection2 { get; set; }
        public ObservableCollection<string> LocationCodeCollection2 { get; set; }

        public string Title { get; set; }
        public bool IsTaskCall { get; set; }
        public bool IsTaskTransport { get; set; }
        public VmLocationProperty Model1 { get; set; }
        public VmLocationProperty Model2 { get; set; }
        public string DialogHostName { get; set; }
        public DelegateCommand SaveCommand { get; set; }
        public DelegateCommand CancelCommand { get; set; }

        public DelegateCommand DropDownOpened1 { get; set; }
        public DelegateCommand LocationCodeSelectionChangedCommand { get; set; }
        public DelegateCommand DropDownOpened2 { get; set; }

        public SimpleTaskViewModel()
        {
            Model1 = new VmLocationProperty();
            Model2 = new VmLocationProperty();

            WareHouseCodeCollection1 = new ObservableCollection<string>();
            LocationCodeCollection1 = new ObservableCollection<string>();
            MaterialIdCollection = new ObservableCollection<string>();

            WareHouseCodeCollection2 = new ObservableCollection<string>();
            LocationCodeCollection2 = new ObservableCollection<string>();


            SaveCommand = new DelegateCommand(Save);
            CancelCommand = new DelegateCommand(Cancel);
            DropDownOpened1 = new DelegateCommand(UpdateLocationCodeCollection1);
            LocationCodeSelectionChangedCommand = new DelegateCommand(LocationCodeSelectionChanged);
            DropDownOpened2 = new DelegateCommand(UpdateLocationCodeCollection2);
        }

        private void UpdateLocationCodeCollection1()
        {
            LocationCodeCollection1.Clear();
            var dataList = locationManager.GetList(d => d.WareHouseCode == Model1.WareHouseCode);

            dataList.ForEach(p => LocationCodeCollection1.Add(p.LocationCode));
            LocationCodeCollection1.Add("");

            MaterialIdCollection.Clear();
            var mDataList = dataList.Where(d => !string.IsNullOrEmpty(d.MaterialId)).Select(d => d.MaterialId).Distinct().ToList();
            mDataList.ForEach(p => MaterialIdCollection.Add(p));
            MaterialIdCollection.Add("");
        }
        private void UpdateLocationCodeCollection2()
        {
            LocationCodeCollection2.Clear();
            var dataList = locationManager.GetList(d => d.WareHouseCode == Model2.WareHouseCode);

            dataList.ForEach(p => LocationCodeCollection2.Add(p.LocationCode));
            LocationCodeCollection2.Add("");
        }
        private void LocationCodeSelectionChanged()
        {
            if (string.IsNullOrEmpty(Model1.LocationCode)) return;

            var data = locationManager.GetById(Model1.LocationCode);
            if(data != null)
            {
                MaterialIdCollection.Clear();
                MaterialIdCollection.Add(data.MaterialId);

                Model1.MaterialId = data.MaterialId;
            }
        }

        /// <summary>
        /// 取消
        /// </summary>
        private void Cancel()
        {
            if (DialogHost.IsDialogOpen(DialogHostName))
                DialogHost.Close(DialogHostName, new DialogResult(ButtonResult.No)); //取消返回NO告诉操作结束
        }

        /// <summary>
        /// 确定
        /// </summary>
        private void Save()
        {
            if (DialogHost.IsDialogOpen(DialogHostName))
            {
                //确定时,把编辑的实体返回并且返回OK
                DialogParameters param = new DialogParameters();
                param.Add("Value1", Model1);
                param.Add("Value2", Model2);
                DialogHost.Close(DialogHostName, new DialogResult(ButtonResult.OK, param));
            }
        }

        public void OnDialogOpend(IDialogParameters parameters)
        {
            if (parameters.ContainsKey("Value1") && parameters.ContainsKey("Value2"))
            {
                Title = parameters.GetValue<string>("Title");
                var data1 = parameters.GetValue<VmLocationProperty>("Value1");
                var data2 = parameters.GetValue<VmLocationProperty>("Value2");

                WareHouseCodeCollection1.Clear();
                LocationCodeCollection1.Clear();
                MaterialIdCollection.Clear();
                WareHouseCodeCollection2.Clear();
                LocationCodeCollection2.Clear();

                IsTaskCall = data1 == null && data2 != null;
                IsTaskTransport = data1 != null && data2 == null;

                if (IsTaskTransport)
                {
                    WareHouseCodeCollection1.Add(data1.WareHouseCode);
                    LocationCodeCollection1.Add(data1.LocationCode);
                    MaterialIdCollection.Add(data1.MaterialId);

                    var arr = data1.LocationCode.GetLocationMember("目标仓库").ToValueArray<string>();
                    for (int i = 0; i < arr.Length; i++)
                    {
                        WareHouseCodeCollection2.Add(arr[i]);
                    }

                    if(arr.Length > 0)
                    {
                        Model2.WareHouseCode = arr[0];
                        UpdateLocationCodeCollection2();
                    }
                }

                if(IsTaskCall)
                {
                    var arr = data2.LocationCode.GetLocationMember("源仓库").ToValueArray<string>();
                    for (int i = 0; i < arr.Length; i++)
                    {
                        WareHouseCodeCollection1.Add(arr[i]);
                    }

                    if (arr.Length > 0)
                    {
                        Model1.WareHouseCode = arr[0];
                        UpdateLocationCodeCollection1();
                    }

                    WareHouseCodeCollection2.Add(data2.WareHouseCode);
                    LocationCodeCollection2.Add(data2.LocationCode);
                }

                data1?.Adapt(Model1);
                data2?.Adapt(Model2);
            }
        }
    }
}
