﻿using DbCommon.Enties.DbModels;
using Prism.Commands;
using Prism.Events;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystemView.Infrastructure;
using ProjectManagementSystemView.Infrastructure.Extensions;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystemView.ViewModels
{
    [AddINotifyPropertyChangedInterface]
    public class Wms2DViewModel : WmsQueryViewModel
    {
        private int sWidth;
        private int sHeight;
        private double multipleX;
        private double multipleY;
        private double offsetX;
        private double offsetY;

        public double GridHeight { get; set; }
        public double GridWidth { get; set; }

        public DelegateCommand ResetLayoutCommand { get; private set; }
        public DelegateCommand SaveLayoutCommand { get; private set; }


        public Wms2DViewModel(IDialogHostService dialogService, IEventAggregator eventAggregator) : base(dialogService, eventAggregator)
        {
            sWidth = AppSetting.TryGetValue<int>("StationUIWidth");
            sHeight = AppSetting.TryGetValue<int>("StationUIHeight");

            ResetLayoutCommand = new DelegateCommand(ResetLayout);
            SaveLayoutCommand = new DelegateCommand(SaveLayout);
        }

        protected override void UpdateVmDatas(List<LocationProperty> dataList)
        {
            var buttonPropertyDict = ExcelConfig.Instance.ButtonPropertyDict;
            var routeConfifDict = ExcelConfig.Instance.RouteConfigDictionary;

            VmDatas.Update(dataList, (s, v) => s.LocationCode == v.LocationCode,
                    (s, v) =>
                    {
                        if (buttonPropertyDict.TryGetValue(v.LocationCode, out CButton cButton))
                        {
                            v.X = cButton.LocationX;
                            v.Y = cButton.LocationY;
                        }
                        else
                        {
                            if (routeConfifDict.TryGetValue(v.LocationCode, out RouteConfig config))
                            {
                                v.X = (Math.Abs(config.X + offsetX) * multipleX) + 5;
                                v.Y = (Math.Abs(config.Y + offsetY) * multipleY) + 5;

                                var cb = new CButton()
                                {
                                    ButtonName = v.LocationCode,
                                    LocationX = v.X,
                                    LocationY = v.Y
                                };
                                buttonPropertyDict.Add(v.LocationCode, cb);
                            }
                        }

                        v.XLength = sWidth;
                        v.YLength = sHeight;

                    });
        }

        private void GetLocationXY()
        {
            var configList = string.IsNullOrEmpty(WareHouseCode) ?
                ExcelConfig.Instance.RouteConfigList.Where(d => !string.IsNullOrEmpty(d.WareHouseCode)) :
                ExcelConfig.Instance.RouteConfigList.Where(d => d.WareHouseCode == WareHouseCode);

            configList = configList.Where(d => d.X != 0 && d.Y != 0);
            if (configList.Count() == 0)
            {
                return;
            }

            double MinX = configList.Min(d => d.X);
            double MaxX = configList.Max(d => d.X);

            double MinY = configList.Min(d => d.Y);
            double MaxY = configList.Max(d => d.Y);

            if (MinX == 0 && MaxX == 0 && MinY == 0 && MaxY == 0)
            {
                return;
            }

            double DistanceX = Math.Abs(MaxX - MinX);
            double DistanceY = Math.Abs(MaxY - MinY);

            if (DistanceX < GridWidth) DistanceX = GridWidth * 10;
            if (DistanceY < GridHeight) DistanceY = GridHeight * 10;

            multipleX = GridWidth * 0.85 / DistanceX;
            multipleY = GridHeight * 0.85 / DistanceY;

            offsetX = 0 - MinX;
            offsetY = 0 - MaxY;
        }


        protected override void Loaded()
        {
            GetLocationXY();
        }
        protected override void ReloadLayout()
        {
            GetLocationXY();
            VmDatas.Clear();
        }
        private void ResetLayout()
        {
            GetLocationXY();

            var buttonPropertyDict = ExcelConfig.Instance.ButtonPropertyDict;
            for (int i = 0; i < VmDatas.Count; i++)
            {
                string locationCode = VmDatas[i].LocationCode;
                if (buttonPropertyDict.ContainsKey(locationCode))
                {
                    buttonPropertyDict.Remove(locationCode);
                }
            }

            VmDatas.Clear();
        }
        private void SaveLayout()
        {
            var buttonPropertyDict = ExcelConfig.Instance.ButtonPropertyDict;

            foreach (var v in VmDatas)
            {
                if (buttonPropertyDict.TryGetValue(v.LocationCode, out CButton cButton)
                && v.DragX > 0
                && v.DragY > 0)
                {
                    var X = v.DragX;
                    var Y = v.DragY - 32;

                    X = X < 0 ? 0 : X;
                    Y = Y < 0 ? 0 : Y;

                    if (v.X == X && v.Y == Y)
                    {
                        continue;
                    }

                    cButton.LocationX = X;
                    cButton.LocationY = Y;
                }
            }

            if (!ExcelConfig.Instance.SaveButtonPropertyList())
            {
                HandyControl.Controls.Growl.Error("保存布局失败");
            }

            VmDatas.Clear();
        }
    }
}
