﻿using DbCommon.Enties.DbModels;
using HelixToolkit.Wpf;
using Prism.Events;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystemView.Infrastructure;
using ProjectManagementSystemView.Infrastructure.Extensions;
using ProjectManagementSystemView.Infrastructure.Models;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace ProjectManagementSystemView.ViewModels
{
    [AddINotifyPropertyChangedInterface]
    public class Wms3DViewModel : WmsQueryViewModel
    {
        public ObservableCollection<VmResultAgvData> VmDatasAgv { get; set; } = new ObservableCollection<VmResultAgvData>();

        public Wms3DViewModel(IDialogHostService dialogService, IEventAggregator eventAggregator) : base(dialogService, eventAggregator)
        {
        }

        protected override void UpdateVmDatas(List<LocationProperty> dataList)
        {
            var routeConfifDict = ExcelConfig.Instance.RouteConfigDictionary;

            VmDatas.Update(dataList, (s, v) => s.LocationCode == v.LocationCode,
                    (s, v) =>
                    {
                        if (v.CX == 0 && v.CX == 0 && routeConfifDict.TryGetValue(v.LocationCode, out RouteConfig config1))
                        {
                            v.CX = config1.X / 1000.0;
                            v.CY = config1.Y / 1000.0;
                        }

                        v.XLength = 0.8;
                        v.YLength = 0.8;

                        if (s.Status == LocationStatus.Empty)
                        {
                            v.ZLength = 0.02;
                        }
                        else
                        {
                            v.ZLength = 0.8;
                        }

                        v.CZ = v.ZLength / 2.0;
                    });

            var agvDataList = ProjectManagementSystem.Common.Service.Crms.PmsApi.GetAllCarrier();
            var agvDataList2 = agvDataList?.Where(d => d.AlarmList.Exists(s => s.alarmId == 75) == false);
            VmDatasAgv.Update(agvDataList2, (s, v) => s.AgvID == v.AgvID,
                (s, v) =>
                {
                    v.CX = s.MovementX / 1000.0;
                    v.CY = s.MovementY / 1000.0;
                    v.CZ = 1.6;
                });
        }
    }
}
